/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#ifndef ALGORITHMS_H
#define ALGORITHMS_H

#include <apt-pkg/depcache.h>

class ProblemResolver;

/*!
 * Fix.Protect() all deps, marked for install, of candidate version of
 * Pkg.
 */
void ProtectInstallDeps(const pkgCache::PkgIterator &Pkg,
			ProblemResolver &Fix);

/*!
 * Change dependencies of candidate version of Pkg to better candidate
 * versions if available.  If a dependency is already installed from a
 * lower priority origin, a candidate from a higher priority origin is
 * selected.
 */
void MarkDepsWithBetterCandidates (const pkgCache::PkgIterator &Pkg,
				   ProblemResolver &Fix);

/*!
 * Mark dependencies of candidate version of Pkg for install.  This
 * allows for noncandidate versions to be installed, not just the
 * candidate versions that pkgDepCache::MarkInstall(AutoInst=true)
 * selects.
 *
 * Modified from pkgDepCache::MarkInstall().
 */
bool ForceDeps(const pkgCache::PkgIterator &Pkg, ProblemResolver &Fix);

#endif
