/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#ifndef APT_WORKER_INTERNAL_H
#define APT_WORKER_INTERNAL_H

#include <glib.h>

#include "package.h"
#include "apt-worker-utils.h"

void
apt_worker_init (DownloadProgressFunc download_progress_fn,
        OperationStartedFunc operation_started_fn,
		OperationProgressFunc operation_progress_fn,
		OperationCompleteFunc operation_complete_fn,
		PackageListUpdatedFunc package_list_updated_fn,
		PackageIgnoredFunc package_ignored_fn,
		gpointer progress_data);

void
apt_worker_free ();

void
apt_worker_reset_state ();

void
cmd_get_package_list (const gchar *category,
		PkgType type,
		const gchar *pattern,
		GList **packages,
		PkgSort sort,
		GError **error);

Package*
cmd_get_file_details (const char *filename, GError **error);

GHashTable *
cmd_get_file_details_map (const char *filename, GError **error);

Package*
cmd_get_package_details (const char *package, const char *version, const char *domain, gboolean installed, GError **error);

GHashTable *
cmd_get_package_details_map (const char *package, const char *version, const char *domain, gboolean installed, GError **error);

GList *
cmd_get_package_dep_details (const char *package,
			     const char *version,
			     GError **error);

gchar*
cmd_get_restore_status(GError **error);

gboolean
cmd_get_restorable_packages(GList **retval, GError **error);

GHashTable *
cmd_get_current_operation_data (GError **error);

GHashTable *
cmd_simulate_operation(const gchar* operation, const gchar* packageidentifier, const gchar* version, const char *domain, const gchar* flags, GError **error);

void
cmd_ignore_update_package (const char *package, const char *version, GError **error);

gboolean
cmd_refresh_cache (const char *only_domain, const char *flags, GError **error);

gboolean
cmd_install_file (const char *filename, GError **error);

gboolean
cmd_install_package (const char *package, const char *version, const char *domain, const char *alt_download_root, GError **error);

gboolean
cmd_upgrade_package (const char *package, const char *version, const char *domain, const char *alt_download_root, GError **error);

gboolean
cmd_restore_package (const char *package, const char *version, const char *domain, GError **error);

void
cmd_ignore_restore_package (const char *package, GError **error);

gboolean
cmd_upgrade_all (gboolean allow_system_upgrade, GError **error);

gboolean
cmd_download_system_upgrade (const char *package, const char *version, const char *domain, const char *alt_download_root, GError **error);

gboolean
cmd_upgrade_system (const char *package, const char *version, const char *domain, const char *alt_download_root, GError **error);

gboolean
cmd_uninstall_package (const char *package, GError **error);

gboolean 
cmd_pkgmgr_fetch_files_of_installed_package (const char *package, GList **files, GError **error);

ResultCode
cmd_check_updates (gboolean with_status, const char *only_domain, bool clear_ignored);

gboolean
cmd_cancel (const char *operation, const char *package, const char *version, GError **error);

gboolean
cmd_get_catalogues (GList **catalogues, GError **error);

gboolean
cmd_add_catalogue (const gchar *name,
		   const gchar *uri,
		   const gchar *dist,
		   const gchar *comp,
		   GError **error);

gboolean
cmd_remove_catalogue (const gchar *name,
		      GError **error);

gboolean
cmd_enable_catalogue (const gchar *name, gboolean enable,
		      GError **error);

#endif // APT_WORKER_INTERNAL_H
