/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#ifndef APT_WORKER_OPERATION_H
#define APT_WORKER_OPERATION_H

#include <glib.h>

G_BEGIN_DECLS

typedef enum _OperationType {
	OPERATION_INSTALL=0,
	OPERATION_INSTALL_FILE,
	OPERATION_UNINSTALL,
	OPERATION_REFRESH,
	OPERATION_UPGRADE,
	OPERATION_UPGRADE_ALL,
	OPERATION_SYSTEM_UPGRADE,
	OPERATION_DOWNLOAD_SYSTEM_UPGRADE,
	OPERATION_RESTORE_INSTALL,
	OPERATION_RESTORE_UPGRADE
} OperationType;

typedef enum _OperationStatus {
	OPERATION_STATUS_CHECKING,
	OPERATION_STATUS_EXECUTING,
	OPERATION_STATUS_SUCCESS,
	OPERATION_STATUS_FAILED,
	OPERATION_STATUS_CANCELLED,
} OperationStatus;

typedef struct OperationData {
	OperationType type;
	gchar *pkg; //!< Package name.
	gchar *version;
	gchar* filename; // This could be empty (i.e. installing from repo)
        gchar *domain;
	int current_bytes;
	int total_bytes;
	int operation_progress_pct;
	const gchar *error;
	GList *installed_user_deps;
	GList *updated_user_deps;
	GList *removed_user_deps;
	GList *pending_pkgs;
	GList *rejected_pkgs;
	int pkgflags;
	gboolean send_signals;
	gboolean send_dep_signals;
	gboolean update_failed_installs;
	gboolean cancel_flag;
        gboolean download_failed_flag;
	gboolean revert_install;
} OperationData;

typedef struct PackageVersion {
	gchar *pkg;
	gchar *version;
        GError *error;
} PackageVersion;

OperationData*
operation_data_new ();

void
operation_data_free (OperationData* od);

OperationData*
operation_data_copy (OperationData* od);

gboolean
need_reboot (int pkgflags);

gboolean
operation_need_reboot (OperationData* od);

gboolean
operation_failed (const OperationData *od);

const gchar*
operation_type_to_string(OperationType type);

GList*
copy_package_version_list (GList *list);

PackageVersion*
package_version_new ();

void
package_version_free (PackageVersion* pv);

PackageVersion*
package_version_copy (PackageVersion* pv);

G_END_DECLS

#endif /* APT_WORKER_OPERATION_H */
