/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#ifndef APT_WORKER_THREAD_H /* -*- c++ -*- */
#define APT_WORKER_THREAD_H

#include <glib.h>
#include "apt-worker-cmd.h"

gboolean apt_worker_thread_default_end_cb (gpointer);

class AptWorkerThread
{
public:
	typedef void (*cmd_cb_t) (apt_worker_cmd *, gpointer);

	AptWorkerThread (AptWorker *apt_worker,
			 GSourceFunc stopped_fn,
			 gpointer stopped_data,
			 DownloadProgressFunc download_progress_fn,
			 OperationStartedFunc operation_started_fn,
			 OperationProgressFunc operation_progress_fn,
			 OperationCompleteFunc operation_complete_fn,
			 PackageListUpdatedFunc package_list_updated_fn,
			 PackageIgnoredFunc package_ignored_fn,
			 gpointer signal_data,
			 gboolean sticky = FALSE);
// 	AptWorkerThread (AptWorker *apt_worker,
// 			 DownloadProgressFunc download_progress_fn,
// 			 OperationStartedFunc operation_started_fn,
// 			 OperationProgressFunc operation_progress_fn,
// 			 gpointer progress_data,
// 			 cmd_cb_t cmd_end_cb, gpointer user_data);
	~AptWorkerThread ();

	void start ();

	void stop ();

	gboolean is_stopping ();
	gboolean is_busy() const { return _cmd_executing || _query_executing; }

	/**
	 * Pushes command to _cmd_q. When cmd_thread_func() has
	 * executed @a cmd, @a callback will be called via
	 * g_idle_add().
	 */
	void push_cmd (apt_worker_cmd *cmd,
		       apt_worker_cmd::callback_t callback);

	/**
	 * Pushes query to _query_q. When query_thread_func() has
	 * executed @a cmd, @a callback will be called via
	 * g_idle_add().
	 */
	void push_query (apt_worker_cmd *cmd,
			 apt_worker_cmd::callback_t callback);

public:
	void cmd_thread_func ();
	void query_thread_func ();

private:
	void set_cmd_executing(bool b);
	void set_query_executing(bool b);

        void start_idle_timer();
        void stop_idle_timer();

	AptWorker *_apt_worker;
	GSourceFunc _stopped_fn;
	gpointer _stopped_data;
	DownloadProgressFunc _download_progress_fn;
	OperationStartedFunc _operation_started_fn;
	OperationProgressFunc _operation_progress_fn;
	OperationCompleteFunc _operation_complete_fn;
	PackageListUpdatedFunc _package_list_updated_fn;
	PackageIgnoredFunc _package_ignored_fn;
	gpointer _signal_data;

// 	cmd_cb_t _cmd_end_cb;
// 	gpointer _cmd_end_cb_user_data;

	GThread *_cmd_worker;
	GAsyncQueue *_cmd_q;
	GThread *_query_worker;
	GAsyncQueue *_query_q;
	gboolean _stopping;
	
	bool _cmd_executing;
	bool _query_executing;
	guint _idle_timer_id;
	GMutex *_state_mutex;

	gboolean _sticky;
};

#endif
