/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include <glib.h>
#include <dbus/dbus-glib-bindings.h>

#include "debug.h"

#include "dbus.h"
#include "pkgmgr-if.h"

PkgMgr* init_dbus(GSourceFunc quit_fn, gpointer quit_data)
{
	PkgMgr *pkgmgr = NULL;
	DBusGConnection* con = NULL;
	GError *error = NULL;
    guint request_ret;
	DBusGProxy* proxy = NULL;
	con = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
	if (error) {
		g_printerr("Failed to connect to system bus: %d: %s\n", error->code, error->message);
		g_error_free(error);
		return NULL;
	}
	pkgmgr = pkgmgr_new();

	proxy = dbus_g_proxy_new_for_name (con,
					DBUS_SERVICE_DBUS,
					DBUS_PATH_DBUS,
					DBUS_INTERFACE_DBUS);
	dbus_g_connection_register_g_object (con,
					PKGMGR_DBUS_PATH,
					G_OBJECT(pkgmgr));

        if (!org_freedesktop_DBus_request_name (proxy,
						PKGMGR_DBUS_SERVICE,
						0,
						&request_ret,
						&error))
	{
                g_printerr("Failed to register service: %d: %s\n", error->code, error->message);
                g_error_free(error);
                return NULL;
        }

	if (request_ret != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER) {
                g_printerr("Failed to own service name %s, daemon already running?\n",
			   PKGMGR_DBUS_SERVICE);
		return NULL;
	}

	return pkgmgr;
}
void deinit_dbus (PkgMgr *pkgmgr)
{
	pkgmgr_stop (pkgmgr);
}
