/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#ifndef DOMAIN_H
#define DOMAIN_H

#include <glib.h>

#include <apt-pkg/sourcelist.h>
#include <apt-pkg/debmetaindex.h>

#include "xexp.h"

#define DOMAIN_INVALID -1

typedef int domain_t;

struct domain_info {
  const char *name;
  xexp *conf;
  int trust_level;
  bool is_certified;
};

void
read_domain_conf ();

domain_t
find_domain_by_tag (const gchar *tag, const gchar *val);

domain_t
find_domain_by_key (const gchar *key);

domain_t
find_domain_by_uri (const gchar *uri);

domain_t
find_domain_by_name (const gchar *name);

void
set_sources_for_get_domain (pkgSourceList *sources);

domain_t
get_domain (pkgIndexFile *index);

domain_t
get_domain (metaIndex *meta);
                
const gchar*
domain_get_name(int n);

int
domain_get_count();

int
domain_get_trust_level(int n);

bool
domain_is_certified(int n);

bool
domain_dominates_or_is_equal (int a, int b);

domain_t
get_file_domain (const char *filename);

void
domain_conf_free ();

#endif /* DOMAIN_H */
