/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include "error-util.h"
#include "serialize.h"
#include <cassert>
#include <cstdlib>

gboolean g_error_fread (GError **pp, FILE *f)
{
  char *line = NULL;
  size_t len = 0;
  int domain;
  gint code;
  gchar *message = 0;
  gboolean ok;

  assert (pp);

  if (get_int (f, &line, &len, &domain)
      && get_int (f, &line, &len, &code)
      && get_str (f, &line, &len, &message))
  {
    if (domain)
      *pp = g_error_new ((GQuark) domain, code, message);
    ok = TRUE;
  }
  else
    ok = FALSE;

  if (line)
    free (line);
  if (message)
    free (message);

  return ok;
}

static gboolean __g_error_fwrite (gint domain, GQuark code, const gchar *message, FILE *f)
{
  return put_int (f, (gint) domain)
    && put_int (f, code)
    && put_str (f, message);
}

gboolean g_error_fwrite (const GError *p, FILE *f)
{
  if (p)
    return __g_error_fwrite (p->domain, p->code, p->message, f);
  else
    return __g_error_fwrite (0, 0, "", f);
}
