/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include "error.h"

const gchar*
pkgmgr_error_to_dbus_error(PkgmgrdError err)
{
	switch(err) {
	case ERROR_INVALID_ARGUMENT:
		return PKGMGR_DBUS_ERROR ".InvalidArgument";
	case ERROR_FILE_NOT_FOUND:
		return PKGMGR_DBUS_ERROR ".FileNotFound";
	case ERROR_PACKAGE_NOT_FOUND:
		return PKGMGR_DBUS_ERROR ".PackageNotFound";
	case ERROR_PACKAGE_VERSION_NOT_FOUND:
		return PKGMGR_DBUS_ERROR ".PackageVersionNotFound";
	case ERROR_INVALID_FILE:
		return PKGMGR_DBUS_ERROR ".InvalidFile";
	case ERROR_INVALID_PACKAGE:
		return PKGMGR_DBUS_ERROR ".InvalidPackage";
	case ERROR_PACKAGE_NOT_COMPATIBLE:
		return PKGMGR_DBUS_ERROR ".PackageNotCompatible";
	case ERROR_MISSING_DEPENDENCIES:
		return PKGMGR_DBUS_ERROR ".MissingDependencies";
	case ERROR_PACKAGE_NOT_USER:
		return PKGMGR_DBUS_ERROR ".PackageNotUser";
	case ERROR_PACKAGE_NOT_INSTALLED:
		return PKGMGR_DBUS_ERROR ".PackageNotInstalled";
	case ERROR_PACKAGE_ALREADY_INSTALLED:
		return PKGMGR_DBUS_ERROR ".PackageAlreadyInstalled";
	case ERROR_PACKAGE_ALREADY_NEWEST:
		return PKGMGR_DBUS_ERROR ".PackageAlreadyNewest";
	case ERROR_CHECK_UPDATES_FAILED:
		return PKGMGR_DBUS_ERROR ".CheckUpdatesFailed";
	case ERROR_NO_CANDIDATE:
		return PKGMGR_DBUS_ERROR ".NoCandidate";
	case ERROR_OPERATION_FAILED:
		return PKGMGR_DBUS_ERROR ".OperationFailed";
	case ERROR_UPGRADE_FAILED:
		return PKGMGR_DBUS_ERROR ".UpgradeFailed";
	case ERROR_CATALOGUE_NOT_FOUND:
		return PKGMGR_DBUS_ERROR ".CatalogueNotFound";
	case ERROR_ACCESS_DENIED:
		return PKGMGR_DBUS_ERROR ".AccessDenied";
	case ERROR_CATALOGUE_ALREADY_EXISTS:
		return PKGMGR_DBUS_ERROR ".CatalogueAlreadyExists";
	case ERROR_SERVICE_SHUTDOWN:
		return PKGMGR_DBUS_ERROR ".ServiceShutdown";
	case ERROR_NOT_ENOUGH_SPACE:
		return PKGMGR_DBUS_ERROR ".NotEnoughSpace";
	case ERROR_NEED_DOWNLOAD:
		return PKGMGR_DBUS_ERROR ".NeedDownload";
	case ERROR_PACKAGE_REQUIRES_SYSTEM_PACKAGE:
		return PKGMGR_DBUS_ERROR ".RequiresSystemPackage";
	case ERROR_PACKAGE_REQUIRES_SYSTEM_UPGRADE:
		return PKGMGR_DBUS_ERROR ".RequiresSystemUpgrade";
	case ERROR_PACKAGE_IS_SYSTEM_PACKAGE:
		return PKGMGR_DBUS_ERROR ".PackageIsSystemPackage";
	case ERROR_DOWNLOAD_FAILED:
		return PKGMGR_DBUS_ERROR ".DownloadFailed";
	case ERROR_NO_OPERATION:
		return PKGMGR_DBUS_ERROR ".NoOperation";
	case ERROR_INVALID_OPERATION:
		return PKGMGR_DBUS_ERROR ".InvalidOperation";
	case ERROR_NOT_CANCELABLE:
		return PKGMGR_DBUS_ERROR ".NotCancelable";
	case ERROR_OPERATION_CANCELED:
		return PKGMGR_DBUS_ERROR ".OperationCanceled";
	case ERROR_DEPENDENCIES_PREVENT_UNINSTALL:
		return PKGMGR_DBUS_ERROR ".DependenciesPreventUninstall";
	case ERROR_DEPENDENCIES_PREVENT_OPERATION:
		return PKGMGR_DBUS_ERROR ".DependenciesPreventOperation";
	case ERROR_NEED_DOWNGRADE:
		return PKGMGR_DBUS_ERROR ".NeedDowngrade";
	case ERROR_LOW_BATTERY_LEVEL:
		return PKGMGR_DBUS_ERROR ".LowBatteryLevel";
	case ERROR_INTERNAL:
		return PKGMGR_DBUS_ERROR ".Internal";
	case ERROR_PARTIAL_SUCCESS:
		return PKGMGR_DBUS_ERROR ".PartialSuccess";
	case ERROR_REMOVED_BY_SYSTEM_UPGRADE:
		return PKGMGR_DBUS_ERROR ".RemovedBySystemUpgrade";
        case ERROR_PACKAGE_NOT_COMPATIBLE_WITH_SYSTEM:
		return PKGMGR_DBUS_ERROR ".PackageNotCompatibleWithSystem";
	default:
		return PKGMGR_DBUS_ERROR ".UnknownError";
	}
}


