/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#ifndef FAILED_INSTALLS_H
#define FAILED_INSTALLS_H

#include "xexp.h"
#include <glib.h>

#define FAILED_INSTALLS_FILE "/var/lib/package-manager/failed-installs"
#define FAILED_UPGRADES_FILE "/var/lib/package-manager/failed-upgrades"

class FailedInstalls
{
public:
  FailedInstalls (const char *filename = FAILED_INSTALLS_FILE);
  ~FailedInstalls ();

  bool read ();
  bool write ();
  bool contains (const char *package);
  const char *failed_version (const char *package);
  const char *failed_domain (const char *package);
  void store (const char *package, const char *version, const char *fname, const char *domain, GError *error);
  bool clear (const char *package);
  bool clear (const char *package, const char *version);
  bool clear_by_filename (const char *filename);
  bool get_data (const char *package, const char *version, const char *&error_code, const char *&error_msg, const char *&filename, const char *&domain);
  bool get_data (const char *package, const char *version, GHashTable *p);
  void reset ();

  /* Return a list with the names of all packages.  The strings in
     that list must not be freed and are only valid as long this
     object is not changed.  The list spine itself must be freed.
  */
  GList *get_packages ();

private:
  const char *filename;
  xexp *installs;
};

#endif
