/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include "format-metadata.h"
#include <cstdio>
#include <cstring>

#define DBG(fmt, ...) g_debug("%s(): " fmt, __func__, ## __VA_ARGS__)

using namespace std;

static bool is_empty_line (const gchar *line, int len)
{
  return len == 1 && line[0] == '.';
}

gchar *format_multiline_field (const gchar *lines, int len, bool join_paragraphs)
{
  if (len == -1)
    len = strlen (lines);

  const gchar *line = lines;

  // Handle first line

  gchar *nl = g_strstr_len (line, len, "\n");
  if (!nl)
    return g_strndup (line, len);

  GString *out = g_string_sized_new (len);
  g_string_append_len (out, line, nl - line);

  // Handle additional lines

  const gchar *end = line + len;
  bool in_paragraph = false;
  line = nl + 1;

  while (line && line < end - 1)
    {
      if (*line != ' ')
	break;

      ++line;
      bool verbatim = line < end && line[0] == ' ';
      if (verbatim)
	++line;

      int n = end - line;
      gchar *nl = g_strstr_len (line, n, "\n");
      len = nl ? nl - line : n;

      bool para = len > 0 && !verbatim && !is_empty_line (line, len);

      if (join_paragraphs && para && in_paragraph)
	{
	  if (line[len - 1] != ' ')
	    g_string_append_c (out, ' ');
	}
      else
	g_string_append_c (out, '\n');

      in_paragraph = para;

      if (!is_empty_line (line, len))
	g_string_append_len (out, line, len);

      line += len + 1;
    }

  return g_string_free (out, FALSE);
}

// convert to utf8
gchar *format_utf8(const gchar* text)
{        
    if(!text) {
        return NULL;
    }
    GString *out = g_string_new("");
    if(g_utf8_validate(text, -1, NULL)) {
        // text is valid utf8 so returning it
        g_string_append(out, text);
    } else {
        // unknown characters found. Seek and replace with '?'.
        gchar* pos = (gchar*)text;
        while(pos && *pos) {
            gunichar cur = g_utf8_get_char_validated(pos, -1);
            if(cur != (gunichar)-1 && cur != (gunichar)-2) {
                gchar cutf8[10];
                int ll = g_unichar_to_utf8(cur, cutf8);
                cutf8[ll] = '\0';
                g_string_append(out, cutf8);
            } else {
                g_string_append_c(out, '?');
            }
            pos = g_utf8_find_next_char (pos, NULL);
        }
    }
    return g_string_free (out, FALSE);

    /*
    if(!text) {
        return NULL;
    }
    gchar *temp = g_locale_to_utf8(text, -1, 0, 0, 0);
    if ( temp ) {
        return g_strdup(temp);
    }
    GString *out = g_string_new("");
    int i;
    const int len = strlen(text);
    for(i=0; i<len; ++i) {
        temp = g_strdup_printf("%c", text[i]);
        if(g_locale_to_utf8(temp, -1, 0, 0, 0)) {
            g_string_append_c(out, text[i]);
        }
        g_free(temp);
    }
    return g_string_free (out, FALSE);
    */
}
