/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#ifndef INST_PKG_TS_STORAGE_H
#define INST_PKG_TS_STORAGE_H

#include <glib.h>
#include <time.h>
#include <string.h>

#define INSTALLED_PACKAGES_FILE "/var/lib/package-manager/installed-packages"

/**
 * This struct contains all meta datas of one package.
 */

struct package_info {
    gchar* package;
    gchar* version;
    time_t timestamp;
    gchar* filename;
};

void clearList(GList* list);

/**
 * A class that handles timestamps of packages.
 */
class PkgTsStorage
{
public:
    PkgTsStorage();
    ~PkgTsStorage();

    bool addPackage(const gchar* package, const gchar* version, time_t timestamp = time(NULL));
    bool addFile(const gchar* package, const gchar* version, const gchar* filename, time_t timestamp = time(NULL));
    bool removePackage(const gchar* package);
    bool removePackage(const gchar* package, const gchar* version);
    bool removeFile(const gchar* filename);
    bool removeFile(const gchar* filename, const gchar* version);

    bool saveToFile(const gchar* filename);
    bool readFromFile(const gchar* filename);

    bool clearStorage();

    time_t getPackageTimestamp(const gchar* package, bool *ok);
    time_t getPackageTimestamp(const gchar* package, const gchar* version, bool *ok);
    gchar* getPackageFilename(const gchar* package, bool *ok);
    gchar* getPackageFilename(const gchar* package, const gchar* version, bool *ok);
    time_t getFileTimestamp(const gchar* filename, bool *ok);
    time_t getFileTimestamp(const gchar* filename, const gchar* version, bool *ok);

    GList *sortedList();
private:
    gconstpointer findPackage(const gchar* package);
    gconstpointer findPackage(const gchar* package, const gchar* version);
    gconstpointer findFile(const gchar* filename);
    gconstpointer findFile(const gchar* filename, const gchar* version);
    GList* copyStorage();
    gchar* packageToString(package_info* pkg);
    package_info* stringToPackage(const gchar* package, bool *ok);
    
    GList *storage;
};

#endif // INST_PKG_TS_STORAGE_H
