/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#ifndef MARKPOLICYDEPCACHE_H
#define MARKPOLICYDEPCACHE_H

#include <apt-pkg/depcache.h>

class MarkPolicyDepCache;

class MarkPolicy
{
public:
    typedef pkgCache::DepIterator DepIterator;
    typedef pkgCache::PkgIterator PkgIterator;
    typedef pkgCache::VerIterator VerIterator;

    MarkPolicy() : DCache(0) { }
    virtual ~MarkPolicy();

    virtual bool IsInstallOk(const PkgIterator &Pkg, bool AutoInst,
			     unsigned long Depth, bool FromUser);
    virtual bool IsDeleteOk(const PkgIterator &Pkg, bool AutoInst,
			    unsigned long Depth, bool FromUser);

    MarkPolicyDepCache *DCache;
};

/*!
 * Custom pkgDepCache that uses MarkPolicy to check if package
 * installations or deletions are allowed.
 */
class MarkPolicyDepCache : public pkgDepCache
{
public:
    MarkPolicyDepCache(pkgCache *Cache, Policy *Plcy = 0,
		       MarkPolicy *markPlcy = 0);
    ~MarkPolicyDepCache();

    void SetMarkPolicy(MarkPolicy *Plcy);

    /*!
     * Reimplementation of pkgDepCache::MarkInstall() that returns
     * false if IsInstallOk() fails.
     *
     * If StopOnError == true, we stop trying to mark more packages.
     * This is allows for faster error detection when the caller is
     * only interested whether all valid dependencies could be
     * satisified.
     */
    bool MarkInstall1(PkgIterator const &Pkg,bool AutoInst = true,
		      unsigned long Depth = 0, bool FromUser = true,
		      bool ForceImportantDeps = false,
		      bool StopOnError = false);

    /*!
     * Reimplementation of pkgDepCache::MarkDelete() that returns
     * false if IsDeleteOk() fails.
     */
    bool MarkDelete1(PkgIterator const &Pkg, bool Purge = false,
		     unsigned long Depth = 0, bool FromUser = true);

    virtual bool IsInstallOk(const PkgIterator &Pkg,bool AutoInst = true,
			     unsigned long Depth = 0, bool FromUser = true);
    virtual bool IsDeleteOk(const PkgIterator &Pkg, bool AutoInst = true,
			    unsigned long Depth = 0, bool FromUser = true);

    MarkPolicy *markPolicy;

 private:
    bool MarkInstall_p(PkgIterator const &Pkg,bool AutoInst = true,
                       unsigned long Depth = 0, bool FromUser = true,
                       bool ForceImportantDeps = false,
                       bool StopOnError = false);

    bool MarkDelete_p(PkgIterator const &Pkg, bool rPurge,
                      unsigned long Depth, bool FromUser);

    int UpdatesNeeded;
    void NeedsUpdate(const PkgIterator &Pkg);
    void DoUpdates();
};

#endif
