/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#ifndef NETCONNECTION_H
#define NETCONNECTION_H

#include <glib.h>
#include <conic.h>
#include <string>

/*!
connect() and libconic connection-event signals:

1) Internet connection: off, USB cable: disconnected

- "Switch internet connection on?": Wait until blank screen
- CON_IC_STATUS_DISCONNECTED(iap_id=null)

2) Internet connection: off, USB cable: disconnected

- "Switch internet connection on?": Yes
- CON_IC_STATUS_DISCONNECTED(iap_id=null)
- "Connect to internet": Select IAP, Connect
- No further connection-event signals

3) Internet connection: on, but disconnected, USB cable: disconnected

- "Connect to internet": Select IAP, Connect
- CON_IC_STATUS_CONNECTED(iap_id)

Disconnect by user:

- CON_IC_STATUS_DISCONNECTING(iap_id)

Disconnect by timeout:

- Wait 

*/
class NetConnection
{
public:
  enum State { Disconnected, Connecting, Connected, Disconnecting };
  enum Event { EvConnecting, EvConnected, EvCancel, EvDisconnecting, EvDisconnected };
  enum ConnectionStatus { StatusDisconnected, StatusConnected, StatusCanceled, StatusTimedOut };

  NetConnection (GMainLoop *loop, gint response_fd, gint status_fd);
  // NetConnection (GMainLoop *loop = 0);
  ~NetConnection ();

  //typedef void (*handler_t) ();
  //void set_disconnecting_handler (handler_t handler) { disconnecting_handler = handler; }
  void connect (bool automatic);
  void disconnect ();

private:
  void transit (State state);
  void set_connection_status (ConnectionStatus status);
  void connection_event (ConIcConnection *connection,
			 ConIcConnectionEvent *event);
  static void static_connection_event (ConIcConnection *connection,
				       ConIcConnectionEvent *event,
				       gpointer user_data);

  bool debug;
  State state;
  std::string iap_id;
  // GMutex *status_mutex;
  // GCond *status_change_cond;
  ConnectionStatus status;
  ConIcConnection *connection;
  GMainLoop *event_loop;
  gint response_fd;
  gint status_fd;
  bool send_response;
  //handler_t disconnecting_handler;
};

#endif
