/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include "package-list.h"
#include "serialize.h"
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

gboolean
package_fread (Package **pp, FILE *f)
{
  char *line = NULL;
  size_t len = 0;
  Package *p = package_new ();
  int type;
  int install_status;
  
  time_t* its = &p->installed_timestamp;
  gint64* itsp = (gint64 *)its;
  time_t* ats = &p->available_timestamp;
  gint64* atsp = (gint64 *)ats;
  GHashTable* extra_fields = 0;
  if (get_str (f, &line, &len, &p->name)
      && get_boolean (f, &line, &len, &p->is_broken)
      && get_str (f, &line, &len, &p->version)
      && get_str (f, &line, &len, &p->filename)
      && get_str (f, &line, &len, &p->category)
      && get_int64 (f, &line, &len, &p->installed_size)
      && get_int64 (f, &line, &len, &p->download_size)
      && get_int64 (f, &line, &len, itsp)
      && get_int64 (f, &line, &len, atsp)
      && get_str (f, &line, &len, &p->display_name)
      && get_str (f, &line, &len, &p->description)
      && get_str (f, &line, &len, &p->repository)
      && get_str (f, &line, &len, &p->icon_data)
      && get_boolean (f, &line, &len, &p->flag_reboot)
      && get_boolean (f, &line, &len, &p->flag_ignored)
      && get_boolean (f, &line, &len, &p->flag_system_update)
      && get_boolean (f, &line, &len, &p->flag_local_file)
      && get_int (f, &line, &len, &type)
      && get_int (f, &line, &len, &install_status)
      && get_str (f, &line, &len, &p->sys_upgrade_name)
      && get_str (f, &line, &len, &p->sys_upgrade_version)
      && get_str (f, &line, &len, &p->error_name)
      && get_str (f, &line, &len, &p->error_message)
      && get_hash_table( f, &line, &len, &extra_fields)
      && getline (&line, &len, f) != -1)
  {
    p->type = (PkgType) type;
    p->install_status = (PkgInstallStatus) install_status;
    p->extra_fields = extra_fields;

/* 		printf ("Name=%s\n", p->name); */

    free (line);

    *pp = p;
    return TRUE;
  }
  else
  {
    free (line);
    package_free (p);
    return FALSE;
  }
}

gboolean
package_fwrite (const Package *p, FILE *f)
{
  return put_str (f, p->name)
    && put_boolean (f, p->is_broken)
    && put_str (f, p->version)
    && put_str (f, p->filename)
    && put_str (f, p->category)
    && put_int64 (f, p->installed_size)
    && put_int64 (f, p->download_size)
    && put_int64 (f, p->installed_timestamp)
    && put_int64 (f, p->available_timestamp)
    && put_str (f, p->display_name)
    && put_str (f, p->description)
    && put_str (f, p->repository)
    && put_str (f, p->icon_data)
    && put_boolean (f, p->flag_reboot)
    && put_boolean (f, p->flag_ignored)
    && put_boolean (f, p->flag_system_update)
    && put_boolean (f, p->flag_local_file)
    && put_int (f, (int) p->type)
    && put_int (f, (int) p->install_status)
    && put_str (f, p->sys_upgrade_name)
    && put_str (f, p->sys_upgrade_version)
    && put_str (f, p->error_name)
    && put_str (f, p->error_message)
    && put_hash_table(f, p->extra_fields)
    && fputs ("\n", f);
}

void
package_list_free (GList *l)
{
  GList *i;

  for (i = (GList *) l; i != NULL; i = g_list_next (i))
    package_free ((Package *) i->data);

  g_list_free (l);
}

gboolean
package_list_fread (GList **lp, FILE *f)
{
  Package *p = NULL;
  GList *l = NULL;

  while (package_fread (&p, f))
    l = g_list_prepend (l, p);

  if (l)
  {
    *lp = g_list_reverse (l);
  }

  return TRUE;
}

gboolean
package_list_fwrite (const GList *l, FILE *f)
{
  GList *i;

  for (i = (GList *) l; i != NULL; i = g_list_next (i))
  {
    package_fwrite ((Package *) i->data, f);
  }

  return TRUE;
}
