/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#ifndef PKGMGR_DBUS_H
#define PKGMGR_DBUS_H

#include <glib-object.h>

#include "package.h"
#include "apt-worker-thread.h"

GType pkgmgr_get_type (void);

typedef struct _PkgMgr {
        GObject parent;
	AptWorkerThread *apt_worker_thread;
} PkgMgr;

typedef struct _PkgMgrClass {
    GObjectClass parent;
    guint download_progress;
    guint operation_started;
    guint operation_progress;
    guint operation_complete;
    guint package_list_updated;
    guint package_ignored;
} PkgMgrClass;

#define PKGMGR_TYPE              (pkgmgr_get_type ())
#define PKGMGR(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), PKGMGR_TYPE, PkgMgr))
#define PKGMGR_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), PKGMGR_TYPE, PkgMgrClass))
#define IS_PKGMGR(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), PKGMGR_TYPE))
#define IS_PKGMGR_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), PKGMGR_TYPE))
#define PKGMGR_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), PKGMGR_TYPE, PkgMgrClass))

PkgMgr *pkgmgr_new ();
void pkgmgr_unref (PkgMgr *obj);
void pkgmgr_start(PkgMgr *obj, GSourceFunc quit_fn, gpointer quit_data, gboolean sticky = FALSE);
void pkgmgr_stop (PkgMgr *obj);


gboolean pkgmgr_fetch_packages(PkgMgr *obj,
			const gchar *category,
			DBusGMethodInvocation *method_context);


gboolean pkgmgr_fetch_available_updates(PkgMgr *obj,
					const gchar *category,
					DBusGMethodInvocation *method_context);


gboolean pkgmgr_fetch_installed(PkgMgr *obj,
				DBusGMethodInvocation *method_context);

gboolean pkgmgr_fetch_package_data_file (PkgMgr *obj,
					const gchar *filename,
					DBusGMethodInvocation *method_context);

gboolean pkgmgr_fetch_package_data (PkgMgr *obj,
				const gchar *packagename,
				const gchar *version,
				DBusGMethodInvocation *method_context);

gboolean pkgmgr_fetch_package_data_installed (PkgMgr *obj,
					const gchar *packagename,
					DBusGMethodInvocation *method_context);

gboolean pkgmgr_fetch_package_dependency_data (PkgMgr *obj,
					       const gchar *packagename,
					       const gchar *version,
					       DBusGMethodInvocation *method_context);

gboolean pkgmgr_fetch_restore_status (PkgMgr *obj,
					       DBusGMethodInvocation *method_context);

gboolean pkgmgr_fetch_restorable_packages (PkgMgr *obj,
					       DBusGMethodInvocation *method_context);

gboolean pkgmgr_fetch_current_operation_data (PkgMgr *obj,
					      DBusGMethodInvocation *method_context);

gboolean pkgmgr_simulate_operation(PkgMgr *obj,
			const gchar *operation,
			const gchar *packageidentifier,
			const gchar *version,
			const gchar *flags,
			DBusGMethodInvocation *method_context);

gboolean pkgmgr_install_file(PkgMgr *obj,
			const gchar *filename,
			DBusGMethodInvocation *method_context);

gboolean pkgmgr_ignore_update_package (PkgMgr *obj,
				const gchar *packagename,
				const gchar *version,
				DBusGMethodInvocation *method_context);

gboolean pkgmgr_refresh_cache (PkgMgr *obj,
			gboolean clearignored,
			DBusGMethodInvocation *method_context);

gboolean pkgmgr_refresh_package_cache (PkgMgr *obj,
                        const gchar *only_domain,
			const gchar *flags,
			DBusGMethodInvocation *method_context);

gboolean pkgmgr_install (PkgMgr *obj,
			const gchar *packagename,
			const gchar *version,
			DBusGMethodInvocation *method_context);

gboolean pkgmgr_upgrade (PkgMgr *obj,
			const gchar *packagename,
			const gchar *version,
			DBusGMethodInvocation *method_context);

gboolean pkgmgr_restore (PkgMgr *obj,
			const gchar *packagename,
			const gchar *version,
			DBusGMethodInvocation *method_context);

gboolean pkgmgr_ignore_restore_package (PkgMgr *obj,
				const gchar *packagename,
				DBusGMethodInvocation *method_context);

gboolean pkgmgr_upgrade_all (PkgMgr *obj,
			     gboolean allow_system_upgrade,
			     DBusGMethodInvocation *method_context);

gboolean pkgmgr_uninstall (PkgMgr *obj,
			const gchar *packagename,
			DBusGMethodInvocation *method_context);

gboolean pkgmgr_fetch_files_of_installed_package (PkgMgr *obj,
                        const gchar *packagename,
                        DBusGMethodInvocation *method_context);

gboolean pkgmgr_download_system_upgrade (PkgMgr *obj,
					 const gchar *packagename,
					 const gchar *version,
					 DBusGMethodInvocation *method_context);

gboolean pkgmgr_upgrade_system (PkgMgr *obj,
				const gchar *packagename,
				const gchar *version,
				DBusGMethodInvocation *method_context);

gboolean pkgmgr_cancel (PkgMgr *obj,
			const gchar *operationid,
			const gchar *package,
			const gchar *version,
			DBusGMethodInvocation *method_context);

gboolean pkgmgr_fetch_catalogues(PkgMgr *obj,
				DBusGMethodInvocation *method_context);

gboolean pkgmgr_add_catalogue(PkgMgr *obj,
			      const gchar *name,
			      const gchar *uri,
			      const gchar *distribution,
			      const gchar *components,
			      DBusGMethodInvocation *method_context);

gboolean pkgmgr_remove_catalogue (PkgMgr *obj,
				  const gchar *name,
				  DBusGMethodInvocation *method_context);

gboolean pkgmgr_enable_catalogue (PkgMgr *obj,
				  const gchar *name,
				  gboolean enable,
				  DBusGMethodInvocation *method_context);

#endif // PKGMGR_DBUS_H
