/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include "debug.h"
#include <conic.h>
#include <cstdlib>
#include <apt-pkg/configuration.h>

static ConIcConnection *connection_object = NULL;

static char *
get_conic_proxy (ConIcProxyProtocol conic_protocol)
{
  ConIcProxyMode proxy_mode;
  const gchar *host;
  gint port;
  gchar *proxy = NULL;

  if (connection_object == NULL)
    connection_object = con_ic_connection_new ();

  proxy_mode = con_ic_connection_get_proxy_mode (connection_object);
  if (proxy_mode == CON_IC_PROXY_MODE_MANUAL)
    {
      host = con_ic_connection_get_proxy_host (connection_object, conic_protocol);
      port = con_ic_connection_get_proxy_port (connection_object, conic_protocol);

      if (host != NULL)
        proxy = g_strdup_printf ("http://%s:%d", host, port);
    }
  else if (proxy_mode == CON_IC_PROXY_MODE_AUTO)
    {
      //! \todo Support CON_IC_PROXY_MODE_AUTO?
    }

  return proxy;
}

static void
setenvi (const char *name, const char *value)
{
  if (value)
    setenv (name, value, 1);
  else
    unsetenv (name);

  DEBUG("%s=%s", name, value ? value : "");
}

void
apply_proxy_settings ()
{
  gchar *proxy;

  unsetenv ("http_proxy");
  unsetenv ("https_proxy");

  if (_config->Find ("Acquire::http::Proxy").empty())
    {
      proxy = get_conic_proxy (CON_IC_PROXY_PROTOCOL_HTTP);
      setenvi ("http_proxy", proxy);
      // _config->Set ("Acquire::http::Proxy", proxy);.
      g_free (proxy);
    }

  if (_config->Find ("Acquire::https::Proxy").empty())
    {
      proxy = get_conic_proxy (CON_IC_PROXY_PROTOCOL_HTTPS);
      setenvi ("https_proxy", proxy);
      // _config->Set ("Acquire::https::Proxy", proxy);.
      g_free (proxy);
    }
}
