/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include <QDebug>

#include "dbus-operators.h"


QDBusArgument &operator<<(QDBusArgument &s, const Package &data)
{
	s.beginMap(QMetaType::QString, qMetaTypeId<QDBusVariant>());
	QMapIterator<QString, QVariant> i(data.p);
	while (i.hasNext()) {
		i.next();
		s.beginMapEntry();
		s << i.key() << QDBusVariant(i.value());
		s.endMapEntry();
	}
	s.endMap();
	return s;
}

const QDBusArgument &operator>>(const QDBusArgument &s, Package &data)
{
	s.beginMap();
	QString key;
	QDBusVariant val;
	while ( !s.atEnd() ) {
		s.beginMapEntry();
		s >> key >> val;
		s.endMapEntry();
		data.p[key] = val.variant();
	}
	s.endMap();
	return s;
}

QDBusArgument &operator<<(QDBusArgument &s, const PackageArray &data)
{
	s.beginArray( qMetaTypeId<Package>() );
	for ( int i = 0; i < data.packages.count(); ++i ) {
		Package pkg;
		pkg.p = data.packages[i];
		s << pkg;
	}
	s.endArray();
	return s;
}

const QDBusArgument &operator>>(const QDBusArgument &s, PackageArray &data)
{
	s.beginArray();
	while ( !s.atEnd() ) {
		Package pkg;
		s >> pkg;
		data.packages.append( pkg.p );
	}
	s.endArray();
	return s;
}

QDBusArgument &operator<<(QDBusArgument &s, const Catalogue &data)
{
	s.beginMap(QMetaType::QString, qMetaTypeId<QDBusVariant>());
	QMapIterator<QString, QVariant> i(data.p);
	while (i.hasNext()) {
		i.next();
		s.beginMapEntry();
		s << i.key() << QDBusVariant(i.value());
		s.endMapEntry();
	}
	s.endMap();
	return s;
}

const QDBusArgument &operator>>(const QDBusArgument &s, Catalogue &data)
{
	s.beginMap();
	QString key;
	QDBusVariant val;
	while ( !s.atEnd() ) {
		s.beginMapEntry();
		s >> key >> val;
		s.endMapEntry();
		data.p[key] = val.variant();
	}
	s.endMap();
	return s;
}

QDBusArgument &operator<<(QDBusArgument &s, const CatalogueArray &data)
{
	s.beginArray( qMetaTypeId<Catalogue>() );
	for ( int i = 0; i < data.catalogues.count(); ++i ) {
		Catalogue pkg;
		pkg.p = data.catalogues[i];
		s << pkg;
	}
	s.endArray();
	return s;
}

const QDBusArgument &operator>>(const QDBusArgument &s, CatalogueArray &data)
{
	s.beginArray();
	while ( !s.atEnd() ) {
		Catalogue pkg;
		s >> pkg;
		data.catalogues.append( pkg.p );
	}
	s.endArray();
	return s;
}
