/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#ifndef DBUSABSTRACTINTERFACEWITHTIMEOUT_H
#define DBUSABSTRACTINTERFACEWITHTIMEOUT_H

#include <QtDBus/QtDBus>

class DBusAbstractInterfaceWithTimeout: public QDBusAbstractInterface
{
public:
  DBusAbstractInterfaceWithTimeout(const QString &service, const QString &path, const char *interface, const QDBusConnection &connection, QObject *parent = 0)
    : QDBusAbstractInterface(service, path, interface, connection, parent),
      callTimeout(-1)
    {
    }

  int callTimeout;

  /*!
    Modified QDBusAbstractInterface::asyncCallWithArgumentList() which
    utilizes callTimeout member variable.

    Doesn't check unaccessible private preconditions
    QDBusAbstractInterfacePrivate::isValid or
    QDBusAbstractInterfacePrivate::canMakeCalls() but they are ok with
    qdbuscpp2xml-generated classes.

    Doesn't set unaccessible private
    QDBusMessagePrivate::setParametersValidated(true) so they are
    unnecessarily checked for slight runtime overhead.
  */
  QDBusPendingCall asyncCallWithArgumentList(const QString& method,
					     const QList<QVariant>& args)
  {
    /* return QDBusAbstractInterface::asyncCallWithArgumentList(method, args); */

    // if (!d->isValid || !d->canMakeCalls()) return
    //     QDBusPendingCall::fromError(d->lastError);

    QDBusMessage msg = QDBusMessage::createMethodCall(service(), path(), interface(), method);
    // QDBusMessagePrivate::setParametersValidated(msg, true);
    msg.setArguments(args);
    return connection().asyncCall(msg, this->callTimeout);
  }
};

#endif
