/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include <QtCore/QObject>
#include <QtCore/QDebug>
#include <QtDBus/QtDBus>
#include <QString>
#include <QVariant>

#include "packagemanager.h"
#include "packagemanager_p.h"
#include "com.nokia.package_manager.h"
#include "package_manager_install_uiproxy.h"
#include "packagemanagerpendingcallwatcher.h"

#define PKGMGR_DBUS_SERVICE "com.nokia.package_manager"
#define PKGMGR_DBUS_PATH "/com/nokia/package_manager"
#define PKGMGR_DBUS_INTERFACE "com.nokia.package_manager"
#define PKGMGR_DBUS_DEFAULT_TIMEOUT (24*60*60*1000) // 24h in milliseconds

PackageManager::PackageManager()
    : QObject( 0 ),
      callTimeout( PKGMGR_DBUS_DEFAULT_TIMEOUT ),
      d_ptr( new PackageManagerPrivate() )
{
    Q_D(PackageManager);

    d->q_ptr = this;

	d->pm = new com::nokia::package_manager(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	QObject::connect(d->pm, SIGNAL(operation_started(const QString&, const QString&, const QString&)),
		this, SIGNAL(operationStarted(const QString&, const QString&, const QString&)));
	QObject::connect(d->pm, SIGNAL(operation_progress(const QString&, const QString&, const QString&, int)),
		this, SIGNAL(operationProgress(const QString&, const QString&, const QString&, int)));
	QObject::connect(d->pm, SIGNAL(operation_complete(const QString&, const QString&, const QString&, const QString&, const bool)),
		this, SIGNAL(operationComplete(const QString&, const QString&, const QString&, const QString&, const bool)));
	QObject::connect(d->pm, SIGNAL(download_progress(const QString&, const QString&, const QString&, int, int)),
		this, SIGNAL(downloadProgress(const QString&, const QString&, const QString&, int, int)));
	QObject::connect(d->pm, SIGNAL(package_list_updated(const bool)),
		this, SIGNAL(packageListUpdated(const bool)));
    QObject::connect(d->pm, SIGNAL(package_ignored(const QString&, const QString&, const QString&)),
		this, SIGNAL(packageIgnored(const QString&, const QString&, const QString&)));

    d->uiProxy = new package_manager_install_uiProxy("com.nokia.package_manager_install_ui",
            "/com/nokia/package_manager_install_ui",
            QDBusConnection::sessionBus(), NULL);
	QObject::connect( d->uiProxy, SIGNAL(operation_aborted(const QString&,const QString&,const QString&,const QString&)),
		this, SIGNAL(operationAborted(const QString&,const QString&,const QString&,const QString&)));
}

PackageManager::~PackageManager()
{
    Q_D(PackageManager);
    if( d ) delete d;
}

PackageManagerPendingFetchWatcher* PackageManager::fetchCurrentOperationData()
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	QDBusPendingReply<Package> reply = am.fetch_current_operation_data();

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher("UpgradeAll", this);
	QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(fetchOneFinishedSlot(QDBusPendingCallWatcher*)));

	return watcher;
}

PackageManagerPendingFetchWatcher* PackageManager::fetchPackages(const QString &category)
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<PackageArray> reply = am.fetch_packages(category);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher("", this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(fetchManyFinishedSlot(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingFetchWatcher* PackageManager::fetchAvailableUpdates(const QString &category)
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<PackageArray> reply = am.fetch_available_updates(category);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher("", this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(fetchManyFinishedSlot(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingFetchWatcher* PackageManager::fetchInstalled()
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<PackageArray> reply = am.fetch_installed();

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher("", this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(fetchManyFinishedSlot(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingFetchWatcher* PackageManager::fetchPackageDataFile(const QString &filename)
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QFileInfo fi(filename);
	QString abs_filename = fi.absoluteFilePath();
	QDBusPendingReply<Package> reply = am.fetch_package_data_file(abs_filename);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(filename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(fetchOneFinishedSlot(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingFetchWatcher* PackageManager::fetchPackageDataRemote(const QString &packagename, const QString& version)
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<Package> reply = am.fetch_package_data( packagename, version );
	
	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(packagename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(fetchOneFinishedSlot(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingFetchWatcher* PackageManager::fetchPackageDataInstalled(const QString &packagename)
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<Package> reply = am.fetch_package_data_installed( packagename );

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(packagename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(fetchOneFinishedSlot(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingFetchWatcher* PackageManager::fetchFilesOfInstalledPackage(const QString &packagename)
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<QStringList> reply = am.fetch_files_of_installed_package( packagename );

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(packagename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(fetchStringListFinishedSlot(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingFetchWatcher* PackageManager::fetchPackageDependencyData(const QString &packagename)
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<PackageArray> reply = am.fetch_package_dependency_data( packagename, NULL );

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(packagename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(fetchManyFinishedSlot(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingFetchWatcher* PackageManager::fetchRestoreStatus()
{
    com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
    am.callTimeout = callTimeout;
    // if (!am.isValid()) {
    // 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
    // 	return;
    // }
    QDBusPendingReply<QString> reply = am.fetch_restore_status();

    PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher("", this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
    QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
            watcher, SLOT(fetchStringFinishedSlot(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingFetchWatcher* PackageManager::fetchRestorablePackages()
{
    com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
    am.callTimeout = callTimeout;
    // if (!am.isValid()) {
    // 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
    // 	return;
    // }
    QDBusPendingReply<PackageArray> reply = am.fetch_restorable_packages();

    PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher("", this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
    QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
            watcher, SLOT(fetchManyFinishedSlot(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingFetchWatcher* PackageManager::simulateOperation(const QString &operation, const QString &packageidentifier, const QString &version, const QString &flags)
{
    com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
    am.callTimeout = callTimeout;
    // if (!am.isValid()) {
    // 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
    // 	return;
    // }
    QDBusPendingReply<Package> reply = am.simulate_operation(operation, packageidentifier, version, flags);

    PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher("", this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
    QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
            watcher, SLOT(fetchOneFinishedSlot(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingCallWatcher* PackageManager::ignoreUpdatePackage( const QString &packagename, const QString &version )
{
    com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<> reply = am.ignore_update_package(packagename, version);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(packagename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingCallWatcher* PackageManager::refreshPackageCache(const QString &onlyDomain, const QString &flags)
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<> reply = am.refresh_package_cache(onlyDomain, flags);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher("", this);
        QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingCallWatcher* PackageManager::installFile(const QString &filename)
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QFileInfo fi(filename);
	QString abs_filename = fi.absoluteFilePath();
	QDBusPendingReply<> reply = am.install_file(abs_filename);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(filename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingCallWatcher* PackageManager::install(const QString &packagename, const QString &version)
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<> reply = am.install(packagename, version);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(packagename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingCallWatcher* PackageManager::upgrade(const QString &packagename, const QString &version)
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<> reply = am.upgrade(packagename, version);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(packagename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingCallWatcher* PackageManager::ignoreRestorePackage( const QString &packagename )
{
    com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<> reply = am.ignore_restore_package(packagename);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(packagename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingCallWatcher* PackageManager::restore(const QString &packagename, const QString &version)
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<> reply = am.restore(packagename, version);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(packagename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingCallWatcher* PackageManager::downloadSystemUpgrade(const QString &packagename, const QString &version)
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<> reply = am.download_system_upgrade(packagename, version);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(packagename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingCallWatcher* PackageManager::upgradeSystem(const QString &packagename, const QString &version)
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<> reply = am.upgrade_system(packagename, version);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(packagename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingCallWatcher* PackageManager::uninstall(const QString &packagename)
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<> reply = am.uninstall(packagename);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(packagename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingCallWatcher* PackageManager::upgradeAll()
{
	com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
	am.callTimeout = callTimeout;
	// if (!am.isValid()) {
	// 	emit dbusError(PKGMGR_DBUS_INTERFACE ".Error.NoService", QString());
	// 	return;
	// }
	QDBusPendingReply<> reply = am.upgrade_all(false);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher("", this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingCallWatcher* PackageManager::cancelOperation(const QString &operation, const QString &packagename, const QString &version)
{
    com::nokia::package_manager am(PKGMGR_DBUS_SERVICE, PKGMGR_DBUS_PATH,QDBusConnection::systemBus(), NULL);
    am.callTimeout = callTimeout;
    QDBusPendingReply<> reply = am.cancel(operation, packagename, version);

    PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher("", this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
    QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
		     watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

// Install UI specific methods

PackageManagerPendingCallWatcher* PackageManager::installFileUi(const QString &filename)
{
    Q_D(PackageManager);
/*	package_manager_install_uiProxy am("com.nokia.package_manager_install_ui",
            "/com/nokia/package_manager_install_ui",
            QDBusConnection::sessionBus(), NULL);*/
    QFileInfo fi(filename);
	QString abs_filename = fi.absoluteFilePath();
	QDBusPendingReply<> reply = d->uiProxy->install_file( abs_filename );

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(filename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingCallWatcher* PackageManager::installPackageUi( const QString &packagename, const QString& version, const QString &origin, const QString &flags )
{
    Q_D(PackageManager);
	/*package_manager_install_uiProxy am("com.nokia.package_manager_install_ui",
            "/com/nokia/package_manager_install_ui",
            QDBusConnection::sessionBus(), NULL);
	QDBusPendingReply<> reply = am.install_package(packagename, version, origin, flags);*/
    QDBusPendingReply<> reply = d->uiProxy->install_package(packagename, version, origin, flags);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(packagename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingCallWatcher* PackageManager::uninstallUi(const QString &packagename)
{
    Q_D(PackageManager);
	/*package_manager_install_uiProxy am("com.nokia.package_manager_install_ui",
            "/com/nokia/package_manager_install_ui",
            QDBusConnection::sessionBus(), NULL);
	QDBusPendingReply<> reply = am.uninstall(packagename);*/
    QDBusPendingReply<> reply = d->uiProxy->uninstall(packagename);


	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(packagename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingCallWatcher* PackageManager::upgradeAllUi()
{
    Q_D(PackageManager);
	/*package_manager_install_uiProxy am("com.nokia.package_manager_install_ui",
            "/com/nokia/package_manager_install_ui",
            QDBusConnection::sessionBus(), NULL);
	QDBusPendingReply<> reply = am.upgrade_all();*/
    QDBusPendingReply<> reply = d->uiProxy->upgrade_all();

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher("", this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

PackageManagerPendingCallWatcher* PackageManager::restoreUi( const QString &packagename, const QString& version )
{
    Q_D(PackageManager);
	/*package_manager_install_uiProxy am("com.nokia.package_manager_install_ui",
            "/com/nokia/package_manager_install_ui",
            QDBusConnection::sessionBus(), NULL);
	QDBusPendingReply<> reply = am.restore(packagename, version);*/
    QDBusPendingReply<> reply = d->uiProxy->restore(packagename, version);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher(packagename, this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}

void PackageManager::showInstallationExceptionUi( const QString &packagename )
{
    Q_D(PackageManager);
    QDBusPendingReply<> reply = d->uiProxy->show_installation_exception(packagename);
}

PackageManagerPendingCallWatcher* PackageManager::refreshPackageCacheUi( const QString &origin, const QString &flags )
{
    Q_D(PackageManager);
    QDBusPendingReply<> reply = d->uiProxy->refresh_package_cache(origin, flags);

	PackageManagerPendingFetchWatcher *watcher = new PackageManagerPendingFetchWatcher("", this);
    QDBusPendingCallWatcher *dbusWatcher = new QDBusPendingCallWatcher(reply, watcher);
	QObject::connect(dbusWatcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
			watcher, SLOT(callFinished(QDBusPendingCallWatcher*)));

    return watcher;
}
void PackageManager::showOperationsPageUi( const QString& operation, const QString &packageidentifier, const QString& version )
{
    QDBusInterface interface( "com.nokia.package_manager_ui", "/com/nokia/package_manager_ui", QString::Null(), QDBusConnection::sessionBus() );
    interface.call( QLatin1String("show_operations_page"), operation, packageidentifier, version );
}

void PackageManager::showUpdatesPageUi()
{
    QDBusInterface interface( "com.nokia.package_manager_ui", "/com/nokia/package_manager_ui", QString::Null(), QDBusConnection::sessionBus() );
    interface.call( QLatin1String("show_updates_page") );
}

void PackageManager::showRestorePageUi()
{
    QDBusInterface interface( "com.nokia.package_manager_ui", "/com/nokia/package_manager_ui", QString::Null(), QDBusConnection::sessionBus() );
    interface.call( QLatin1String("show_restore_page") );
}

void PackageManager::showDetailsPageUi( const QString& type, const QString& packagename, const QString& version )
{
    QDBusInterface interface( "com.nokia.package_manager_ui", "/com/nokia/package_manager_ui", QString::Null(), QDBusConnection::sessionBus() );
    interface.call( QLatin1String("show_details_page"), type, packagename, version );
}

void PackageManager::showInstalledPageUi( const QString& packageidentifier )
{
    QDBusInterface interface( "com.nokia.package_manager_ui", "/com/nokia/package_manager_ui", QString::Null(), QDBusConnection::sessionBus() );
    interface.call( QLatin1String("show_installed_page"), packageidentifier );
}

PackageManagerPrivate::PackageManagerPrivate()
    : QObject( 0 ),
    pm(0),
    uiProxy(0)
{
	qDBusRegisterMetaType<Package>();
	qDBusRegisterMetaType<PackageArray>();
	qDBusRegisterMetaType<Catalogue>();
	qDBusRegisterMetaType<CatalogueArray>();
}

PackageManagerPrivate::~PackageManagerPrivate()
{
	if( pm ) delete pm;
    if( uiProxy ) delete uiProxy;
}

#define ERROR_CODE(error)						\
    else if (errorName == "com.nokia.package_manager.Error." #error)	\
	return PackageManager::Error ## error

PackageManager::ErrorCode PackageManagerPrivate::errorNameToCode( const QString& errorName )
{
    qDebug() << "PackageManagerPrivate::errorNameToCode()" << errorName;
    // Package Manager specific errors
    if( errorName.isEmpty() )
        return PackageManager::ErrorNone;
    ERROR_CODE(Internal);
    ERROR_CODE(InvalidArgument);
    ERROR_CODE(CheckUpdatesFailed);
    ERROR_CODE(FileNotFound);
    ERROR_CODE(InvalidFile);
    ERROR_CODE(PackageNotUser);
    ERROR_CODE(NoCandidate);
    ERROR_CODE(PackageNotFound);
    ERROR_CODE(PackageNotInstalled);
    ERROR_CODE(PackageNotCompatible);
    ERROR_CODE(DependenciesPreventUninstall);
    ERROR_CODE(MissingDependencies);
    ERROR_CODE(DownloadFailed);
    ERROR_CODE(OperationFailed);
    ERROR_CODE(UpgradeFailed);
    ERROR_CODE(PackageAlreadyInstalled);
    ERROR_CODE(MissingDependencies);
    ERROR_CODE(PackageAlreadyNewest);
    ERROR_CODE(InvalidOperation);
    ERROR_CODE(NoOperation);
    ERROR_CODE(NotCancelable);
    ERROR_CODE(OperationCanceled);
    ERROR_CODE(PackageVersionNotFound);
    else
        return PackageManager::ErrorUnknown;
}

