/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include <QDebug>
#include <QString>
#include "packagemanagerpendingcallwatcher.h"
#include "packagemanager_p.h"
#include "com.nokia.package_manager.h"

PackageManagerPendingCallWatcher::PackageManagerPendingCallWatcher( const QString& packageidentifier, QObject * parent )
: QObject(parent),
_packageidentifier(packageidentifier),
isError(false),
finished(false)
{
}

PackageManagerPendingCallWatcher::~PackageManagerPendingCallWatcher()
{

}

QString PackageManagerPendingCallWatcher::packageidentifier()
{
    return _packageidentifier;
}

PackageManager::ErrorCode PackageManagerPendingCallWatcher::errorCode()
{
	if( isError ) 
    {
        qDebug() << "PackageManagerPendingCallWatcher::errorCode()" << _errorType << _errorName << _errorMessage;
        QString prefixTrolltech = "com.trolltech.QtDBus.Error";
        QString prefixFreedesktop = "com.trolltech.QtDBus.Error";
        if( _errorType == QDBusError::NoError )
            return PackageManager::ErrorNone;
        else if( _errorType != QDBusError::Other) 
            return PackageManager::ErrorDbus;
        else
        {
            return PackageManagerPrivate::errorNameToCode( _errorName );
/*
            // Package Manager specific errors
            if( _errorName == "com.nokia.package_manager.Error.Internal" )
                return PackageManager::ErrorInternal;
            else if( _errorName == "com.nokia.package_manager.Error.CheckUpdatesFailed" )
                return PackageManager::ErrorCheckUpdatesFailed;
            else if( _errorName == "com.nokia.package_manager.Error.FileNotFound" )
                return PackageManager::ErrorFileNotFound;
            else if( _errorName == "com.nokia.package_manager.Error.InvalidFile" )
                return PackageManager::ErrorInvalidFile;
            else if( _errorName == "com.nokia.package_manager.Error.PackageNotUser" )
                return PackageManager::ErrorPackageNotUser;
            else if( _errorName == "com.nokia.package_manager.Error.NoCandidate" )
                return PackageManager::ErrorNoCandidate;
            else if( _errorName == "com.nokia.package_manager.Error.PackageNotFound" )
                return PackageManager::ErrorPackageNotFound;
            else if( _errorName == "com.nokia.package_manager.Error.PackageNotInstalled" )
                return PackageManager::ErrorPackageNotInstalled;
            else if( _errorName == "com.nokia.package_manager.Error.PackageNotCompatible" )
                return PackageManager::ErrorPackageNotCompatible;
            else if( _errorName == "com.nokia.package_manager.Error.MissingDependencies" )
                return PackageManager::ErrorMissingDependencies;
            else if( _errorName == "com.nokia.package_manager.Error.OperationFailed" )
                return PackageManager::ErrorOperationFailed;
            else if( _errorName == "com.nokia.package_manager.Error.PackageAlreadyInstalled" )
                return PackageManager::ErrorPackageAlreadyInstalled;
            else if( _errorName == "com.nokia.package_manager.Error.MissingDependencies" )
                return PackageManager::ErrorMissingDependencies;
            else if( _errorName == "com.nokia.package_manager.Error.PackageAlreadyNewest" )
                return PackageManager::ErrorPackageAlreadyNewest;
            else return PackageManager::ErrorUnknown;
*/
        }
    }    
    return PackageManager::ErrorNone;
}

QString PackageManagerPendingCallWatcher::errorName()
{
    return _errorName;
}

QString PackageManagerPendingCallWatcher::errorMessage()
{
    return _errorMessage;
}

bool PackageManagerPendingCallWatcher::isFinished()
{
    return finished;
}

void PackageManagerPendingCallWatcher::callFinished( QDBusPendingCallWatcher* self )
{
    finished = true;
	QDBusPendingReply<> reply = *self;
	if (reply.isError()) 
    {
		QDBusError const& e = reply.error();
        _errorType = e.type();
        _errorName = e.name();
        _errorMessage = e.message();
        isError = true;
        emit dbusError( this );
    }
    emit callFinished(this);
}

PackageManagerPendingFetchWatcher::PackageManagerPendingFetchWatcher( const QString& packageidentifier, QObject * parent )
    : PackageManagerPendingCallWatcher( packageidentifier, parent )
{
}

PackageManagerPendingFetchWatcher::~PackageManagerPendingFetchWatcher()
{
}

void PackageManagerPendingFetchWatcher::fetchOneFinishedSlot( QDBusPendingCallWatcher* self )
{
    finished = true;
	QDBusPendingReply<Package> reply = *self;
	if (reply.isError()) 
    {
		QDBusError const& e = reply.error();
        _errorType = e.type();
        _errorName = e.name();
        _errorMessage = e.message();
        isError = true;
        emit dbusError( this );    
    }
    else
    {
        Package pkg = reply.value();
	    emit dataFetched(pkg.p);
    }
}

void PackageManagerPendingFetchWatcher::fetchManyFinishedSlot( QDBusPendingCallWatcher* self )
{
    finished = true;
	QDBusPendingReply<PackageArray> reply = *self;
	if (reply.isError()) 
    {
		QDBusError const& e = reply.error();
        _errorType = e.type();
        _errorName = e.name();
        _errorMessage = e.message();
        isError = true;
        emit dbusError(this);
    }
    else
    {
    	PackageArray pkgs = reply.value();
	    QList< QMap<QString, QVariant> > packages;
    	for ( int i = 0; i < pkgs.packages.count(); ++i ) 
        {
	    	packages.append(pkgs.packages[i]);
    	}
    	emit dataFetched(packages);
    }
}

void PackageManagerPendingFetchWatcher::fetchStringListFinishedSlot( QDBusPendingCallWatcher* self )
{
    finished = true;
	QDBusPendingReply<QStringList> reply = *self;
	if (reply.isError()) 
    {
		QDBusError const& e = reply.error();
        _errorType = e.type();
        _errorName = e.name();
        _errorMessage = e.message();
        isError = true;
        emit dbusError(this);    
    }
    else
    {
    	QStringList files = reply.value();
	    emit dataFetched(files);
    }
}

void PackageManagerPendingFetchWatcher::fetchStringFinishedSlot( QDBusPendingCallWatcher* self )
{
    finished = true;
	QDBusPendingReply<QString> reply = *self;
	if (reply.isError()) 
    {
		QDBusError const& e = reply.error();
        _errorType = e.type();
        _errorName = e.name();
        _errorMessage = e.message();
        isError = true;
        emit dbusError(this);    
    }
    else
    {
    	QString data = reply.value();
	    emit dataFetched(data);
    }
}


