/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include <QCoreApplication>
#include <QStringList>
#include <QTextStream>

#include "qtcli.h"

int main(int argc, char *argv[])
{
    QCoreApplication app(argc, argv);
    QStringList args = app.arguments();
    QTextStream cout(stdout);

    if (argc == 1) {
        cout << "Usage: qtcli <command> <options>" << endl; 
        cout << "Commands:" << endl;
        cout << "  list-installed <timeout>" << endl;
        cout << "  list-updates <timeout>" << endl;
        cout << "  list-available <timeout>" << endl;
        cout << "  install-file <filename> <timeout>" << endl;
        cout << "  install-file-ui <filename> <timeout>" << endl;
        cout << "  file-details <filename> <timeout>" << endl;
        cout << "  package-details <packagename> <version> <timeout>" << endl;
        cout << "  package-dep-details <packagename> <timeout>" << endl;
        cout << "  list-package <packagename> <timeout>" << endl;
        cout << "  install <packagename> <timeout>" << endl;
        cout << "  install-ui <packagename> <timeout>" << endl;
        cout << "  uninstall <packagename> <timeout>" << endl;
        cout << "  uninstall-ui <packagename> <timeout>" << endl;
        cout << "  upgrade <packagename> <timeout>" << endl;
        cout << "  upgrade-ui <packagename> <timeout>" << endl;
        cout << "  upgrade-all <timeout>" << endl;
        cout << "  upgrade-all-ui <timeout>" << endl;
        cout << "  download-system-upgrade <packagename> <timeout>" << endl;
        cout << "  upgrade-system <packagename> <timeout>" << endl;
        cout << "If timeout is not given, 10000 milliseconds is used as timeout." << endl;
        return 1;
    }

    QtCli qtcli;
    QObject::connect(&qtcli, SIGNAL(done()), &app, SLOT(quit()));

    if (args[1] == "list-installed") {
        if (argc == 3) qtcli.setTimeout(argv[2]);
        qtcli.fetchInstalled();
    }
    else if (args[1] == "list-updates") {
        if (argc == 3) qtcli.setTimeout(argv[2]);
        qtcli.fetchAvailableUpdates();
    }
    else if (args[1] == "list-available") {
        if (argc == 3) qtcli.setTimeout(argv[2]);
        qtcli.fetchPackages();
    }
    else if (args[1] == "install-file" && argc > 2) {
        if (argc == 4) qtcli.setTimeout(argv[3]);
        qtcli.installFile(args[2]);
    }
    else if (args[1] == "install-file-ui" && argc > 2) {
        if (argc == 4) qtcli.setTimeout(argv[3]);
        qtcli.installFileUi(args[2]);
    }
    else if (args[1] == "file-details" && argc > 2) {
        if (argc == 4) qtcli.setTimeout(argv[3]);
        qtcli.fetchPackageDataFile(args[2]);
    }
    else if (args[1] == "package-details" && argc > 3) {
        if (argc == 5) qtcli.setTimeout(argv[4]);
        qtcli.fetchPackageData(args[2], args[3]);
    }
    else if (args[1] == "package-dep-details" && argc > 2) {
        if (argc == 4) qtcli.setTimeout(argv[3]);
        qtcli.fetchPackageDependencyData(args[2]);
    }
    else if (args[1] == "list-package" && argc > 2) {
        if (argc == 4) qtcli.setTimeout(argv[3]);
        qtcli.fetchFilesOfInstalledPackage(args[2]);
    }
    else if (args[1] == "install" && argc > 2) {
        if (argc == 4) qtcli.setTimeout(argv[3]);
        qtcli.install(args[2]);
    }
    else if (args[1] == "install-ui" && argc > 2) {
        if (argc == 4) qtcli.setTimeout(argv[3]);
        qtcli.installUi(args[2]);
    }
    else if (args[1] == "uninstall" && argc > 2) {
        if (argc == 4) qtcli.setTimeout(argv[3]);
        qtcli.uninstall(args[2]);
    }
    else if (args[1] == "uninstall-ui" && argc > 2) {
        if (argc == 4) qtcli.setTimeout(argv[3]);
        qtcli.uninstallUi(args[2]);
    }
    else if (args[1] == "upgrade" && argc > 2) {
        if (argc == 4) qtcli.setTimeout(argv[3]);
        qtcli.upgrade(args[2]);
    }
    else if (args[1] == "upgrade-ui" && argc > 2) {
        if (argc == 4) qtcli.setTimeout(argv[3]);
        qtcli.upgradeUi(args[2]);
    }
    else if (args[1] == "upgrade-all") {
        if (argc == 3) qtcli.setTimeout(argv[2]);
        qtcli.upgradeAll();
    }
    else if (args[1] == "upgrade-all-ui") {
        if (argc == 3) qtcli.setTimeout(argv[2]);
        qtcli.upgradeAllUi();
    }
    else if (args[1] == "download-system-upgrade" && argc > 2) {
        if (argc == 4) qtcli.setTimeout(argv[3]);
        qtcli.downloadSystemUpgrade(args[2]);
    }
    else if (args[1] == "upgrade-system" && argc > 2) {
        if (argc == 4) qtcli.setTimeout(argv[3]);
        qtcli.upgradeSystem(args[2]);
    }
    else {
        cout << "Invalid command" << endl;
        return 1;
    }
    return app.exec();
}

