/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include <QDebug>
#include <QTimer>

#include "qtcli.h"
#include "packagemanager.h"
#include "packagemanagerpendingcallwatcher.h"

QtCli::QtCli() : QObject()
{
    system("source /tmp/session_bus_address.user");
    am = new PackageManager();
    timer = new QTimer(this);
    timer->setInterval(10000);
    connect(timer, SIGNAL(timeout()), this, SLOT(timeout()));

    connect(am, SIGNAL(operationComplete(QString,QString,QString,QString,bool)), this, SLOT(operationComplete(QString,QString,QString,QString,bool)));
    connect(am, SIGNAL(downloadProgress(QString,QString,QString,int,int)), this, SLOT(downloadProgress(QString,QString,QString,int,int)));
    connect(am, SIGNAL(operationProgress(QString,QString,QString,int)), this, SLOT(operationProgress(QString,QString,QString,int)));
    connect(am, SIGNAL(packageListUpdated()), this, SLOT(packageListUpdated()));
}

QtCli::~QtCli()
{
    //dtor
}

void QtCli::fetchInstalled()
{
    qDebug() << "Fetching installed packages";
    timer->start();
    PackageManagerPendingFetchWatcher* watcher = am->fetchInstalled();
    connect(watcher, SIGNAL(dataFetched(QList<QMap<QString,QVariant>>)), this, SLOT(dataFetched(QList<QMap<QString,QVariant>>)));
    connect(watcher, SIGNAL(dbusError(PackageManagerPendingCallWatcher*)), this, SLOT(dbusError(PackageManagerPendingCallWatcher*)));
}

void QtCli::fetchAvailableUpdates()
{
    qDebug() << "Fetching available updates";
    timer->start();
    PackageManagerPendingFetchWatcher* watcher = am->fetchAvailableUpdates();
    connect(watcher, SIGNAL(dataFetched(QList<QMap<QString,QVariant>>)), this, SLOT(dataFetched(QList<QMap<QString,QVariant>>)));
    connect(watcher, SIGNAL(dbusError(PackageManagerPendingCallWatcher*)), this, SLOT(dbusError(PackageManagerPendingCallWatcher*)));
}

void QtCli::fetchPackages()
{
    qDebug() << "Fetching available packages";
    timer->start();
    PackageManagerPendingFetchWatcher* watcher = am->fetchPackages();
    connect(watcher, SIGNAL(dataFetched(QList<QMap<QString,QVariant>>)), this, SLOT(dataFetched(QList<QMap<QString,QVariant>>)));
    connect(watcher, SIGNAL(dbusError(PackageManagerPendingCallWatcher*)), this, SLOT(dbusError(PackageManagerPendingCallWatcher*)));
}

void QtCli::fetchPackageDataFile(const QString &filename)
{
    qDebug() << "Fetching data of package file" << filename;
    timer->start();
    PackageManagerPendingFetchWatcher* watcher = am->fetchPackageDataFile(filename);
    connect(watcher, SIGNAL(dataFetched(QMap<QString,QVariant>)), this, SLOT(dataFetched(QMap<QString,QVariant>)));
    connect(watcher, SIGNAL(dbusError(PackageManagerPendingCallWatcher*)), this, SLOT(dbusError(PackageManagerPendingCallWatcher*)));
}

void QtCli::fetchPackageData(const QString &packagename, const QString& version)
{
    qDebug() << "Fetching data of package" << packagename;
    timer->start();
    PackageManagerPendingFetchWatcher* watcher = am->fetchPackageDataRemote(packagename, version);
    connect(watcher, SIGNAL(dataFetched(QMap<QString,QVariant>)), this, SLOT(dataFetched(QMap<QString,QVariant>)));
    connect(watcher, SIGNAL(dbusError(PackageManagerPendingCallWatcher*)), this, SLOT(dbusError(PackageManagerPendingCallWatcher*)));
}

void QtCli::fetchPackageDependencyData(const QString &packagename)
{
    qDebug() << "Fetching dependency data of package" << packagename;
    timer->start();
    PackageManagerPendingFetchWatcher* watcher = am->fetchPackageDependencyData(packagename);
    connect(watcher, SIGNAL(dataFetched(QList<QMap<QString,QVariant>>)), this, SLOT(dataFetched(QList<QMap<QString,QVariant>>)));
    connect(watcher, SIGNAL(dbusError(PackageManagerPendingCallWatcher*)), this, SLOT(dbusError(PackageManagerPendingCallWatcher*)));
}

void QtCli::fetchFilesOfInstalledPackage(const QString &packagename)
{
    qDebug() << "Fetching files of package" << packagename;
    timer->start();
    PackageManagerPendingFetchWatcher* watcher = am->fetchFilesOfInstalledPackage(packagename);
    connect(watcher, SIGNAL(dataFetched(QStringList)), this, SLOT(dataFetched(QStringList)));
    connect(watcher, SIGNAL(dbusError(PackageManagerPendingCallWatcher*)), this, SLOT(dbusError(PackageManagerPendingCallWatcher*)));
}

void QtCli::installFile(const QString &filename )
{
    qDebug() << "Installing file" << filename;
    timer->start();
    am->installFile(filename);
}

void QtCli::installFileUi(const QString &filename )
{
    qDebug() << "Installing file" << filename << "with install ui";
    timer->start();
    am->installFileUi(filename);
}

void QtCli::install(const QString &packagename)
{
    qDebug() << "Installing" << packagename;
    timer->start();
    am->install(packagename);
}

void QtCli::installUi(const QString &packagename)
{
    qDebug() << "Installing" << packagename << "with install ui";
    timer->start();
    am->installPackageUi(packagename);
}

void QtCli::uninstall(const QString &packagename)
{
    qDebug() << "Uninstalling" << packagename;
    timer->start();
    am->uninstall(packagename);
}

void QtCli::uninstallUi(const QString &packagename)
{
    qDebug() << "Uninstalling" << packagename << "with install ui";
    timer->start();
    am->uninstallUi(packagename);
}

void QtCli::upgrade(const QString &packagename)
{
    qDebug() << "Upgrading" << packagename;
    timer->start();
    am->upgrade(packagename);
}

void QtCli::upgradeUi(const QString &packagename)
{
    qDebug() << "Upgrading" << packagename << "with install ui";
    timer->start();
    am->installPackageUi(packagename);
}

void QtCli::upgradeAll()
{
    qDebug() << "Upgrading all packages";
    timer->start();
    am->upgradeAll();
}

void QtCli::upgradeAllUi()
{
    qDebug() << "Upgrading all packages with install ui";
    timer->start();
    am->upgradeAllUi();
}

void QtCli::downloadSystemUpgrade(const QString &packagename)
{
    qDebug() << "Downloading system upgrade" << packagename;
    timer->start();
    am->downloadSystemUpgrade(packagename);
}

void QtCli::upgradeSystem(const QString &packagename)
{
    qDebug() << "Upgrading system" << packagename;
    timer->start();
    am->upgradeSystem(packagename);
}

void QtCli::setTimeout(QString interval)
{
    bool ok;
    int inter = interval.toInt (&ok);
    if (ok) timer->setInterval(inter);
    qDebug() << "Using" << timer->interval() << "milliseconds as timeout";
}

void QtCli::dataFetched(const QList<QMap<QString, QVariant> >& packageList)
{
    qDebug() << "Fetched" << packageList.count() << "packages";
    for( int i = 0; i < packageList.count(); i++ )
    {
        QMapIterator<QString, QVariant> iter(packageList[i]);
        QString name = NULL;
        QString version = NULL;
        while( iter.hasNext() )
        {
            iter.next();
            if( !iter.key().compare("Name", Qt::CaseInsensitive) )
                name = iter.value().toString();
            if( !iter.key().compare("Version", Qt::CaseInsensitive) )
                version = iter.value().toString();
            if (name != NULL && version != NULL)
                qDebug() << name << version;
        }
    }
    emit done();
}

void QtCli::dataFetched(const QMap<QString, QVariant>& package)
{
    QMapIterator<QString, QVariant> iter(package);
    while( iter.hasNext() )
    {
        iter.next();
        qDebug() << iter.key() << ":" << iter.value().toString(); 
    }
    emit done();
}

void QtCli::dataFetched(const QStringList& fileList)
{
    for( int i = 0; i < fileList.count(); i++ )
    {
        qDebug() << fileList[i];
    }
    emit done();
}

void QtCli::operationComplete(const QString &operation, const QString &pkg, const QString &version, const QString &result, const bool need_reboot)
{
    //TODO: done emitted and program closed after first completed upgrade when performing upgrade all
    Q_UNUSED(need_reboot);
    qDebug() << "Operation" << operation << "for package" << pkg << "version" << version <<"was completed with result" << result;
    emit done();
}

void QtCli::downloadProgress(const QString &operation, const QString &pkg, const QString &version, int already, int total)
{
    qDebug() << "Download progress. Related to operation: " << operation << "Package:" << pkg << "Version:" << version << already << "/" << total;
}

void QtCli::operationStarted(const QString &operation, const QString &packagename, const QString &version)
{
    qDebug() << "Operation" << operation << "for package" << packagename << "version" << version << "started";
}

void QtCli::operationProgress(const QString &op, const QString &pkg, const QString &version, int percentage)
{
    qDebug() << "Operation progress" << op << pkg << version << percentage;
}

void QtCli::packageListUpdated()
{
    qDebug() << "Package list updated!";
}

void QtCli::dbusError(PackageManagerPendingCallWatcher* watcher)
{
    qDebug() << "Error occurred:" << watcher->errorName();
    emit done();
}

void QtCli::timeout()
{
    qDebug() << "Timeout! Operation was not finished in" << timer->interval() << "milliseconds.";
    emit done();
}

