/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#ifndef QtCli_H
#define QtCli_H

#include <QObject>
#include <QMap>

class PackageManager;
class QTimer;
class PackageManagerPendingCallWatcher;

class QtCli : public QObject
{
    Q_OBJECT

public:
    QtCli();
    virtual ~QtCli();
    void fetchInstalled();
    void fetchAvailableUpdates();
    void fetchPackages();
    void fetchPackageDataFile(const QString &filename);
    void fetchPackageData(const QString &packagename, const QString& version);
    void fetchPackageDependencyData(const QString &packagename);
    void fetchFilesOfInstalledPackage(const QString &packagename);
    //void fetchPackageDataInstalled(const QString &packagename);
    void installFile(const QString &filename );
    void installFileUi(const QString &filename );
    void install(const QString &packagename);
    void installUi(const QString &packagename);
    void uninstall(const QString &packagename);
    void uninstallUi(const QString &packagename);
    void upgrade(const QString &packagename);
    void upgradeUi(const QString &packagename);
    void upgradeAll();
    void upgradeAllUi();
    void downloadSystemUpgrade(const QString &packagename);
    void upgradeSystem(const QString &packagename);
    void setTimeout(QString interval);

public slots:
    void dataFetched(const QList<QMap<QString, QVariant> >& packageList);
    void dataFetched(const QMap<QString, QVariant>& package);
    void dataFetched(const QStringList& fileList);
    void operationComplete(const QString &operation, const QString &pkg, const QString &version, const QString &result, const bool need_reboot);
    void downloadProgress(const QString &operation, const QString &pkg, const QString &version, int already, int total);
    void operationStarted(const QString &operation, const QString &packagename, const QString &version);
    void operationProgress(const QString &op, const QString &pkg, const QString &version, int percentage);
    void packageListUpdated();
    void dbusError(PackageManagerPendingCallWatcher* watcher);
    void timeout();

signals:
    void done();

private:
    PackageManager* am;
    QTimer* timer;
};

#endif //QtCli_H

