/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */


#include <glib-object.h>

#include "package.h"

Package* package_new()
{
	return g_new0(Package,1);
}

static gboolean
free_map_value (gpointer key,
	gpointer value,
	gpointer user_data)
{
	gchar *k = (gchar*)key;
	GValue *v = (GValue*)value;
	g_free (k);
	g_value_unset (v);
	g_free (v);
	return TRUE;
}

static void
free_map(GHashTable *map)
{
    if ( map ) {
        g_hash_table_foreach_steal (map, free_map_value, NULL);
        g_hash_table_destroy(map);
    }
}

void package_free(Package *p)
{
	if (p != NULL) {
		g_free(p->name);
		g_free(p->version);
		g_free(p->filename);
		g_free(p->category);
		g_free(p->display_name);
		g_free(p->description);
		g_free(p->repository);
		g_free(p->icon_data);
		g_free(p->sys_upgrade_name);
		g_free(p->sys_upgrade_version);
		g_free(p->error_name);
		g_free(p->error_message);
		free_map(p->extra_fields);
		g_free(p);
	}
}

const gchar*
package_type_to_string(PkgType type)
{
	const gchar *s = "Unknown";
	switch(type) {
	case PKG_TYPE_NOT_INSTALLED:
		s = "NotInstalled";
		break;
	case PKG_TYPE_INSTALLED:
		s = "Installed";
		break;
	case PKG_TYPE_UPDATE:
		s = "Update";
		break;
        case PKG_TYPE_NOT_AVAILABLE:
		s = "NotAvailable";
		break;
	case PKG_TYPE_OLDER_THAN_INSTALLED:
		s = "OlderThanInstalled";
		break;
	case PKG_TYPE_UNKNOWN:
		s = "Unknown";
		break;
	}
	return s;
}

const gchar*
package_install_status_to_string(PkgInstallStatus status)
{
	const gchar *s;
	switch (status) {
	case status_installed:
		s = "Installed";
		break;
	case status_installable:
		s = "Installable";
		break;
	case status_uninstallable:
		s = "Uninstallable";
		break;
	case status_conflicting:
		s = "Conflicting";
		break;
	case status_missing:
		s = "Missing";
		break;
	case status_needed:
		s = "Needed";
		break;
	case status_corrupted:
		s = "Corrupted";
		break;
	case status_incompatible:
		s = "Incompatible";
		break;
	case status_incompatible_section:
		s = "IncompatibleSection";
		break;
	case status_incompatible_current:
		s = "IncompatibleCurrent";
		break;
	case status_system_update_unremovable:
		s = "SystemUpdateUnremovable";
		break;
	case status_not_found:
		s = "NotFound";
		break;

	case status_unknown:
	default:
		s = "Unknown";
		break;		
	}
	return s;
}

