/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#ifndef PACKAGE_H
#define PACKAGE_H

#include <glib.h>
#include <time.h>

G_BEGIN_DECLS
typedef enum _PkgInstallStatus {
  status_unknown = 0,
  status_installed,
  status_installable,
  status_uninstallable,                   // unknown reason
  status_conflicting,
  status_missing,
  status_needed,
  status_corrupted,
  status_incompatible,             // incompatible in general
  status_incompatible_section,     // not a user package
  status_incompatible_current,     // incompatible with current OS
  status_system_update_unremovable,// could be removed but it's a bad idea
  status_not_found,                // there is no such package
} PkgInstallStatus;

typedef enum _PkgType {
	PKG_TYPE_UNKNOWN = 0, /* < unknown*/
	PKG_TYPE_INSTALLED = 1, /* < installed*/
	PKG_TYPE_UPDATE    = 2, /* < installed and new version available*/
	PKG_TYPE_NOT_INSTALLED = 4, /* < available but not installed*/
	PKG_TYPE_OLDER_THAN_INSTALLED = 8, /* < older than installed version*/
	PKG_TYPE_NOT_AVAILABLE = 16, /* < not available*/
} PkgType;


typedef struct _Package {
	gchar *name;
	gboolean is_broken;
	gchar *version;
	gchar *category;
	gint64 installed_size;
	gint64 download_size;
	time_t installed_timestamp;
	time_t available_timestamp;
	gchar *display_name;
	gchar *description;
	gchar *repository;
	gchar *icon_data;
	gboolean flag_reboot;
	gboolean flag_ignored;
	gboolean flag_system_update;
        gboolean flag_local_file;
	gchar* filename;
	PkgType type;
	PkgInstallStatus install_status;
  	gchar *sys_upgrade_name;
	gchar *sys_upgrade_version;
	gchar *error_name;
	gchar *error_message;
	GHashTable *extra_fields;
} Package;

typedef enum _InstallFlags {
  pkgflag_close_apps       =  1,
  pkgflag_suggest_backup   =  2,
  pkgflag_reboot           =  4,
  pkgflag_system_update    =  8,
  pkgflag_flash_and_reboot = 16,
  pkgflag_reboot_update    = 32
} InstallFlags;

typedef enum _ResultCode {
  rescode_success,              // (success)
  rescode_partial_success,
  rescode_cancelled,
  rescode_failure,              // Operation failed
  rescode_download_failed,      // Download failed
  rescode_package_corrupted,    // Package corrupted
  rescode_packages_not_found,   // Unable to download.  The
                                // package was not found.
  rescode_out_of_space          // Not enough memory in target location
} ResultCode;



Package* package_new();

void package_free(Package *p);

const gchar*
package_type_to_string(PkgType type);

const gchar*
package_install_status_to_string(PkgInstallStatus status);

G_END_DECLS

#endif // PACKAGE_H
