/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

/*
  Tool to manipulate Package Manager update availability
  notifications.
 */
#include <MApplication>
#include <MLocale>
#include <MNotification>
#include <QDebug>
#include <QFile>
#include <getopt.h>
#include <cstdio>
#include "notification.h"

static const char *argv0;
static bool recreate = false;
static bool onlyUpdate = false;
static bool print = false;
static int nUpgrades = 0;
static const char *sysUpgrade = 0;
const char *bodyText = "";

static void print_usage()
{
    fprintf(stderr,
	    "usage: %s [OPTION...]\n\n"
	    "Options:\n"
	    "  -c, --recreate         recreate existing notification\n"
	    "  -o, --only-update      update notification if it exists\n"
	    "  -p, --print            print notifications\n"
	    "  -t, --body-text=T      use T as body text\n"
	    "  -u, --upgrades=N       N upgrades available\n"
	    "  -U, --sys-upgrade=P=V  system upgrade P version V available\n",
	    argv0);
}

static void parse_args(int &argc, char **&argv)
{
    for (;;)
    {
	static struct option opts[] =
	{
	    {"create", no_argument, 0, 'c'},
	    {"only-update", no_argument, 0, 'o'},
	    {"print", no_argument, 0, 'p'},
	    {"body-text", required_argument, 0, 't'},
	    {"upgrades", required_argument, 0, 'u'},
	    {"sys-upgrade", no_argument, 0, 'U'},
	    {0, 0, 0, 0}
	};
	int opti = 0;

	int c = getopt_long(argc, argv, "copt:u:U:", opts, &opti);

	if (c == -1)
	    break;

	switch (c)
	{
	case 'c':
	    recreate = true;
	    break;

	case 'o':
	    onlyUpdate = true;
	    break;

	case 'p':
	    print = true;
	    break;

	case 't':
	    bodyText = optarg;
	    break;

	case 'u':
	    nUpgrades = atoi(optarg);
	    break;

	case 'U':
	    sysUpgrade = optarg;
	    break;

	default:
	    print_usage();
	    exit(2);
	}
    }

    argc -= optind;
    argv += optind;
}

static void printNotifications()
{
    QFile file;
    file.open(stdout, QIODevice::WriteOnly);
    QDebug cout(&file);

    QList<MNotification *> N = MNotification::notifications();

    foreach (MNotification *n, N)
    {
	cout << n->eventType() << n->summary() << n->body() << n->count() << "\n";
    }
}

int main(int argc, char *argv[])
{
    argv0 = argv[0];
    alarm(10);

    MApplication app(argc, argv, "pm-notification-tool");

    MLocale locale;
    locale.installTrCatalog("packagemanager");
    MLocale::setDefault(locale);

    parse_args(argc, argv);

    if (print)
	printNotifications();
    else
    {
	updateSystemUpdateNotification(sysUpgrade, QString::fromUtf8(bodyText), sysUpgrade, recreate);
	updateNotification(nUpgrades, recreate, onlyUpdate);
    }

    alarm(0);

    return 0;
}
