/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include "notification.h"
#include <MNotification>
#include <QDebug>
#include <QSettings>

static MNotification *findNotificationByEventType(const QString &eventType)
{
    QList<MNotification *> N = MNotification::notifications();

    foreach (MNotification *n, N)
    {
	if (n->eventType() == eventType)
	    return n;
    }

    return 0;
}

void updateSystemUpdateNotification(bool available, const QString &summaryText, const QString &packageVersion, bool recreate)
{
    static const char *eventType = "x-nokia.system-update";
    static const char *iconId = "icon-m-content-system-update";

    MNotification *n = findNotificationByEventType(eventType);
    if (n && (!available || recreate))
    {
	n->remove();
	delete n;
	n = 0;

	qDebug() << "Removed system upgrade notification";
    }

    if (available)
    {
	//% "Device Update"
	QString body = qtTrId("qtn_pmgr_osupd_notification");
       QString summary(summaryText);

	if (!recreate && n
           && n->body() == body
           && n->summary() == summary
	    && n->identifier() == packageVersion)
	    return; // no-op

	if (!n)
	    n = new MNotification(eventType, summary, body);
	else
	{
	    n->setBody(body);
           n->setSummary(summary);
	}

	n->setIdentifier(packageVersion);

	QString methodName;
	QList<QVariant> args;
	if (!packageVersion.isEmpty())
	{
	    methodName = "show_details_page";
	    args.append("upgrade");
	    QStringList pv = packageVersion.split('=');
	    if (!pv.empty())
		args.append(pv.takeFirst());
	    if (!pv.empty())
		args.append(pv.takeFirst());
	}
	else
	    methodName = "show_updates_page";

	MRemoteAction action("com.nokia.package_manager_ui",
			     "/com/nokia/package_manager_ui",
			     "com.nokia.package_manager_ui",
			     methodName,
			     args);

	n->setAction(action);
	n->publish();

	qDebug() << "Published system upgrade notification:" << eventType << body << summary << iconId << methodName << args;
    }
}

void updateNotification(int count, bool recreate, bool onlyUpdate)
{
    static const char *eventType = "x-nokia.software-update";
    static const char *iconId = "icon-m-content-third-party-update";

    MNotification *n = findNotificationByEventType(eventType);
    if (!n && onlyUpdate && !recreate)
	return;

    if (n && (count == 0 || recreate))
    {
	n->remove();
	delete n;
	n = 0;

	qDebug() << "Removed upgrade notification";
    }

    if (count > 0)
    {
	//% "%Ln Software update"
	QString body = qtTrId("qtn_pmgr_upd_notification", count);

	if (!recreate && n
	    && n->count() == count
	    && n->body() == body)
	    return; // no-op

	if (!n)
	    n = new MNotification(eventType, "", body);
	else
           n->setBody(body);

	n->setCount(count);

	MRemoteAction action("com.nokia.package_manager_ui",
			     "/com/nokia/package_manager_ui",
			     "com.nokia.package_manager_ui",
			     "show_updates_page");

	n->setAction(action);
	n->publish();

	qDebug() << "Published upgrade notification:" << eventType << body << iconId;
    }
}
