/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#ifndef NOTIFICATION_H
#define NOTIFICATION_H

#include <QString>

/**
 * Updates notification about system update.
 *
 * \param available If set, notification will be created; otherwise
 * removed.
 * \param summaryText Notification summary text.
 * \param packageVersion System upgrade package (format name=version).
 * \param recreate Recreate the notification, resulting in transient
 * popup message.
 */
void updateSystemUpdateNotification(bool available = false, const QString &summaryText = QString(), const QString &packageVersion = QString(), bool recreate = false);

/**
 * Updates notification about regular updates.
 *
 * \param count If > 0, notification will be created; otherwise
 * removed.
 * \param recreate Recreate the notification, resulting in transient
 * popup message.
 * \param onlyUpdate Update the notification if it exists, otherwise
 * do nothing.
 */
void updateNotification(int count = 0, bool recreate = false, bool onlyUpdate = false);

#endif
