/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include "testpage.h"
#include "notification.h"
#include <QDebug>
#include <MAction>
#include <MLayout>
#include <MLinearLayoutPolicy>
#include <MNotification>
#include <MRemoteAction>

TestPage::TestPage()
    : layout(0),
      layoutPolicy(0)
{
}

TestPage::~TestPage()
{
}

void TestPage::createContent()
{
    puts(__PRETTY_FUNCTION__);
    MApplicationPage::createContent();

    setTitle("Notifier");
    setEscapeMode(MApplicationPageModel::EscapeCloseWindow);

    layout = new MLayout(centralWidget());
    centralWidget()->setLayout(layout);
    layoutPolicy = new MLinearLayoutPolicy(layout, Qt::Vertical);

    MAction *action = new MAction("icon-m-toolbar-search", "Add system upgrade notification", centralWidget());
    action->setLocation(MAction::ToolBarLocation);
    addAction(action);
    QObject::connect(action, SIGNAL(triggered()),
		     this, SLOT(notifySystemUpgrade()));

    action = new MAction("icon-m-toolbar-search", "Add 1 upgrade notification", centralWidget());
    action->setLocation(MAction::ToolBarLocation);
    addAction(action);
    QObject::connect(action, SIGNAL(triggered()),
		     this, SLOT(notify1RegularUpgrade()));

    action = new MAction("icon-m-toolbar-search", "Add upgrade notification", centralWidget());
    action->setLocation(MAction::ToolBarLocation);
    addAction(action);
    QObject::connect(action, SIGNAL(triggered()),
		     this, SLOT(notifyRandRegularUpgrades()));

    action = new MAction("icon-m-toolbar-search", "Remove notifications", centralWidget());
    action->setLocation(MAction::ToolBarLocation);
    addAction(action);
    QObject::connect(action, SIGNAL(triggered()),
		     this, SLOT(removeNotifications()));
};

#define ENTER() qDebug("%s: enter\n", __PRETTY_FUNCTION__)
#define EXIT() qDebug("%s: exit\n", __PRETTY_FUNCTION__)

void TestPage::notifySystemUpgrade()
{
    updateSystemUpdateNotification(true, "", "", true);
}

void TestPage::notify1RegularUpgrade()
{
    updateNotification(1, true);
}

void TestPage::notifyRandRegularUpgrades()
{
    int n = rand() % 10 + 1;
    updateNotification(n, true);
}

void TestPage::removeNotifications()
{
    updateSystemUpdateNotification();
    updateNotification();
}
