/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include <check.h>
#include <cstdio>
#include <cstdlib>
#include "check-common.h"
#include "format-metadata.h"

struct pair { const gchar *a; const gchar *b; };
static const pair data[] =
  {
    // One line
    { "a",
      "a" },
    // One line, strip newline
    { "a\n",
      "a" },
    // Two lines
    { "a\n"
      " b",
      "a\n"
      "b" },
    // Join paragraphs, not verbatim
    { "a\n"
      " b\n"
      " c\n"
      "  verbatim\n"
      "  verbatim",
      "a\n"
      "b c\n"
      "verbatim\n"
      "verbatim"},
    // Empty line formatting
    { "a\n"
      " b\n"
      " .\n"
      " c",
      "a\n"
      "b\n"
      "\n"
      "c" },
  };

START_TEST (test_format_metadata)
{
  gchar *f = format_multiline_field (data[_i].a, -1, true);
  ck_assert_str_eq (f, data[_i].b);
  g_free (f);
}
END_TEST

START_TEST (test_formatting)
{
  const char *data = "A long synopsis line of hpm-user-test-metadata to test the displaying it in the user interface\n Lorem ipsum\n .\n Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do\n eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut\n enim ad minim veniam, quis nostrud exercitation ullamco laboris\n nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in\n reprehenderit in voluptate velit esse cillum dolore eu fugiat\n nulla pariatur. Excepteur sint occaecat cupidatat non proident,\n sunt in culpa qui officia deserunt mollit anim id est laborum.\n  How about some bullets:\n  * Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed\n  * do eiusmod tempor incididunt ut labore et dolore magna\n  * aliqua. Ut enim ad minim veniam, quis nostrud exercitation\n  * ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis\n  * aute irure dolor in reprehenderit in voluptate velit esse\n  * cillum dolore eu fugiat nulla pariatur. Excepteur sint\n  * occaecat cupidatat non proident, sunt in culpa qui officia\n  * deserunt mollit anim id est laborum.\n .\n  Homepage: <URL:http://www.testanembeddedurl.com/>";
  const char *formatted =
    "A long synopsis line of hpm-user-test-metadata to test the displaying it in the user interface\n"
    "Lorem ipsum\n"
    "\n"
    "Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.\n"
    "How about some bullets:\n"
    "* Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed\n"
    "* do eiusmod tempor incididunt ut labore et dolore magna\n"
    "* aliqua. Ut enim ad minim veniam, quis nostrud exercitation\n"
    "* ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis\n"
    "* aute irure dolor in reprehenderit in voluptate velit esse\n"
    "* cillum dolore eu fugiat nulla pariatur. Excepteur sint\n"
    "* occaecat cupidatat non proident, sunt in culpa qui officia\n"
    "* deserunt mollit anim id est laborum.\n"
    "\n"
    "Homepage: <URL:http://www.testanembeddedurl.com/>";

  gchar *f = format_multiline_field (data, -1, true);
  ck_assert_str_eq (f, formatted);
  g_free (f);
}
END_TEST

static Suite *
create_suite ()
{
  Suite *s = suite_create ("format-metadata");
  TCase *tc;

  tc = tcase_create ("main");
  const int n = sizeof(data) / sizeof(pair);
  tcase_add_loop_test (tc, test_format_metadata, 0, n);
  tcase_add_test (tc, test_formatting);
  suite_add_tcase (s, tc);

  return s;
}

int main (int argc, char *argv[])
{
  // g_type_init ();
  setenv ("G_DEBUG", "fatal_warnings", 1);

  fork_mode = 0;

  return run_suite (create_suite ()) ? EXIT_SUCCESS : EXIT_FAILURE;
}
