/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include "mock-apt-worker.h"
#include <glib.h>

static MockAptWorker *mock = 0;

void
mock_apt_worker_init (DownloadProgressFunc download_progress_fn,
              OperationStartedFunc operation_started_fn,
		      OperationProgressFunc operation_progress_fn,
		      OperationCompleteFunc operation_complete_fn,
		      PackageListUpdatedFunc package_list_updated_fn,
		      PackageIgnoredFunc package_ignored_fn,
		      gpointer progress_data)
{
}

void
mock_apt_worker_reset_state ()
{
}

void
mock_cmd_get_package_list (const gchar *category,
			   PkgType type,
			   const gchar *pattern,
			   GList **packages,
			   PkgSort sort,
			   GError **error)
{
	mock->category = category;
	mock->type = type;
	mock->pattern = pattern;
	mock->sort = sort;

	*packages = mock->retval_packages;

	g_set_error(error, 0, 42, __func__);
}

Package*
mock_cmd_get_file_details (const char *filename, GError **error)
{
	mock->filename = filename;
	g_set_error(error, 0, 42, __func__);

	return mock->retval_pkg_list_info;
}

GHashTable *
mock_cmd_get_file_details_map (const char *filename, GError **error)
{
	mock->filename = filename;
	g_set_error(error, 0, 42, __func__);

	return mock->retval_pkg_map;
}

GList *
mock_cmd_get_package_dep_details (const char *package,
				  const char *version,
				  GError **error)
{
	mock->package = package;
	mock->version = version;
	g_set_error(error, 0, 42, __func__);

	return mock->retval_packages;
}

gchar *
mock_cmd_get_restore_status (GError **error)
{
	g_set_error(error, 0, 42, __func__);

	return mock->retval_status;
}

gboolean
mock_cmd_get_restorable_packages (GList **retval, GError **error)
{
	g_set_error(error, 0, 42, __func__);
    *retval = mock->retval_packages;

	return mock->retval_boolean;
}

GHashTable *
mock_cmd_get_package_details_map (const char *package, const char *version, const char *domain, gboolean installed, GError **error)
{
	mock->package = package;
	mock->version = version;
	mock->installed = installed;
	g_set_error(error, 0, 42, __func__);

	return mock->retval_pkg_map;
}

gboolean
mock_cmd_install_file (const char *filename, GError **error)
{
	mock->filename = filename;
	g_set_error(error, 0, 42, __func__);

	return mock->retval_boolean;
}

void
mock_cmd_ignore_update_package (const char *package, const char *version, GError **error)
{
    mock->package = package;
	mock->version = version;
	g_set_error(error, 0, 42, __func__);
}

gboolean
mock_cmd_refresh_cache (const char *only_domain, const char *flags, GError **error)
{
        mock->flags = flags;
	g_set_error(error, 0, 42, __func__);

	return mock->retval_boolean;
}

gboolean
mock_cmd_install_package (const char *package, const char *version, const char *domain, const char *alt_download_root, GError **error)
{
	mock->package = package;
	mock->version = version;
	mock->alt_download_root = alt_download_root;
	g_set_error(error, 0, 42, __func__);

	return mock->retval_boolean;
}

gboolean
mock_cmd_upgrade_package (const char *package, const char *version, const char *domain, const char *alt_download_root, GError **error)
{
	mock->package = package;
	mock->version = version;
	mock->alt_download_root = alt_download_root;
	g_set_error(error, 0, 42, __func__);
	return mock->retval_boolean;
}

gboolean
mock_cmd_restore_package (const char *package, const char *version, const char *domain, GError **error)
{
	mock->package = package;
	mock->version = version;
	g_set_error(error, 0, 42, __func__);

	return mock->retval_boolean;
}

void
mock_cmd_ignore_restore_package (const char *package, GError **error)
{
    mock->package = package;
	g_set_error(error, 0, 42, __func__);
}

gboolean
mock_cmd_upgrade_all (gboolean allow_system_upgrade, GError **error)
{
	g_set_error(error, 0, 42, __func__);
	return mock->retval_boolean;
}

gboolean
mock_cmd_download_system_upgrade (const char *package, const char *version, const char *domain, const char *alt_download_root, GError **error)
{
	mock->package = package;
	mock->version = version;
	mock->alt_download_root = alt_download_root;
	g_set_error(error, 0, 42, __func__);
	return mock->retval_boolean;
}

gboolean
mock_cmd_upgrade_system (const char *package, const char *version, const char *domain, const char *alt_download_root, GError **error)
{
	mock->package = package;
	mock->version = version;
	mock->alt_download_root = alt_download_root;
	g_set_error(error, 0, 42, __func__);
	return mock->retval_boolean;
}

gboolean
mock_cmd_uninstall_package (const char *package, GError **error)
{
	mock->package = package;
	g_set_error(error, 0, 42, __func__);

	return mock->retval_boolean;
}
gboolean
mock_cmd_pkgmgr_fetch_files_of_installed_package (const char *package, GList **files, GError **error)
{
        mock->package = package;
        g_set_error(error, 0, 42, __func__);

        return mock->retval_boolean;
}

ResultCode
mock_cmd_check_updates (gboolean with_status, const char *only_domain, bool clear_ignored)
{
	mock->with_status = with_status;

	return mock->retval_result_code;
}

gboolean
mock_cmd_get_catalogues (GList **catalogues, GError **error)
{
	*catalogues = mock->retval_catalogues;
	g_set_error(error, 0, 42, __func__);
	return mock->retval_boolean;
}

gboolean
mock_cmd_add_catalogue (const gchar *name,
			const gchar *uri,
			const gchar *dist,
			const gchar *comp,
			GError **error)
{
	mock->name = name;
	g_set_error(error, 0, 42, __func__);
	return mock->retval_boolean;
}

gboolean
mock_cmd_remove_catalogue (const gchar *name, GError **error)
{
	mock->name = name;
	g_set_error(error, 0, 42, __func__);
	return mock->retval_boolean;
}

gboolean
mock_cmd_enable_catalogue (const gchar *name, gboolean enable, GError **error)
{
	mock->name = name;
	mock->enable = enable;
	g_set_error(error, 0, 42, __func__);
	return mock->retval_boolean;
}

GHashTable *
mock_cmd_get_current_operation_data (GError **error)
{
	GHashTable *map = g_hash_table_new (g_str_hash, g_str_equal);
	return map;
}

GHashTable *
mock_cmd_simulate_operation (const gchar *operation, const gchar *packageidentifier, const gchar *version, const char *domain, const gchar *flags, GError **error)
{
	GHashTable *map = g_hash_table_new (g_str_hash, g_str_equal);
	return map;
}

gboolean
mock_cmd_cancel (const gchar *operation, const char *package, const char *version, GError **error)
{
	g_set_error(error, 0, 42, __func__);
	return mock->retval_boolean;
}

static MockAptWorker mock_apt_worker = {
	base: {
		mock_apt_worker_init,
		mock_apt_worker_reset_state,
		mock_cmd_get_package_list,
		mock_cmd_get_file_details,
		mock_cmd_get_file_details_map,
		mock_cmd_get_package_details_map,
		mock_cmd_get_package_dep_details,
		mock_cmd_get_restore_status,
		mock_cmd_get_restorable_packages,
        mock_cmd_ignore_update_package,
		mock_cmd_refresh_cache,
		mock_cmd_get_current_operation_data,
		mock_cmd_simulate_operation,
		mock_cmd_install_file,
		mock_cmd_install_package,
		mock_cmd_upgrade_package,
		mock_cmd_restore_package,
		mock_cmd_ignore_restore_package,
		mock_cmd_download_system_upgrade,
		mock_cmd_upgrade_system,
		mock_cmd_upgrade_all,
		mock_cmd_uninstall_package,
		mock_cmd_pkgmgr_fetch_files_of_installed_package,
		mock_cmd_check_updates,
		mock_cmd_cancel,
		mock_cmd_get_catalogues,
		mock_cmd_add_catalogue,
		mock_cmd_remove_catalogue,
		mock_cmd_enable_catalogue
	},
	0
};

AptWorker *
mock_apt_worker_new ()
{
	mock = &mock_apt_worker;

	return &mock_apt_worker.base;
}
