#!/bin/sh
#
# Test update notification.

set -e
cd `dirname $0`

DELAY=$1
VERBOSITY=1
PKGMGR=../cli/pkgmgr

if test -n "$_SBOX_DIR"; then
    if test -z "$DISPLAY"; then
	export DISPLAY=:1
    fi
    meego-sb-session -quiet start
    . /tmp/session_bus_address.user
fi

run() {
    local out=`mktemp`
    set +e
    $* >$out 2>&1
    set -e
    if test $? -ne 0; then
	echo -n "$* failed with code $?: "
	cat $out
	rm $out
	exit $?
    fi
    rm $out
}

remove_notification() {
    run ../notify/pm-notification-tool -r
}

setup_testrepo() {
    remove_notification
    run ./setup-testrepo.sh
    remove_notification
    test -f /var/lib/package-manager/available-updates && rm $_
    test -f /var/lib/package-manager/available-packages && rm $_
    ./start_pkgmgrd.sh
    $PKGMGR refresh-cache
}

install() {
    ./stop_pkgmgrd.sh
    run apt-get -y --force-yes install $*
    ./start_pkgmgrd.sh
}

n_notifications() {
    mnotificationtool -l pm-notification-tool 2>/dev/null | tail +2 | awk '/^pm-notification-tool has [0-9]* notifications./ {print $3}'
}

fail_if() {
    local line=$1
    shift
    if test $*; then
	echo "$0:$line: Fail: $*" >/dev/stderr
	exit 1
    fi
}

# No upgrades, no notification
setup_testrepo
fail_if $LINENO $(n_notifications) -ne 0

# Nothing installed, no notification
./update-testrepo.sh
$PKGMGR refresh-cache
fail_if $LINENO $(n_notifications) -ne 0

# New system upgrade, notification
install hpm-system-meta=0.1.0-1 hpm-system1=0.1.0-1 hpm-system2=0.1.0-1 hpm-system3=0.1.0-1
$PKGMGR refresh-cache
sleep $DELAY
fail_if $LINENO $(n_notifications) -ne 1

# No new upgrades, no notification
remove_notification
$PKGMGR refresh-cache
fail_if $LINENO $(n_notifications) -ne 0

# Also one user upgrade, notification
$PKGMGR install -p hpm-user1 -v 0.1.0-1
sleep $DELAY
fail_if $LINENO $(n_notifications) -ne 1

# Another user upgrade, notification
$PKGMGR install -p hpm-user2 -v 0.1.0-1
sleep $DELAY
fail_if $LINENO $(n_notifications) -ne 1

# Only user upgrade, notification
$PKGMGR download-system-upgrade -p hpm-system-meta
$PKGMGR upgrade-system -p hpm-system-meta
sleep $DELAY
fail_if $LINENO $(n_notifications) -ne 1

# No upgrades, no notification
$PKGMGR upgrade -p hpm-user1
$PKGMGR upgrade -p hpm-user2
sleep $DELAY
fail_if $LINENO $(n_notifications) -ne 0
