#!/bin/sh
#
# Test that proxy settings are correctly used by pkgmgrd.

source testvars.sh

unset_proxy() {
    keys="proxy_http proxy_http_port proxy_https proxy_https_port proxytype"
    for k in $keys; do
	conn_settings -c -k $k -u
    done
}

dbus_send() {
    method=$1
    shift
    dbus-send --system \
	--type=method_call \
	--print-reply \
	--dest='com.nokia.package_manager' \
	/com/nokia/package_manager \
	com.nokia.package_manager.$method \
	$*
}

trap unset_proxy EXIT

PROXY_HOST=127.0.0.1
PROXY_PORT=9999

stop_pkgmgrd.sh
set +e
rm /var/cache/apt/archives/hpm-*.deb
set -e
conn_settings -c -k proxy_http -p -t string -- $PROXY_HOST
conn_settings -c -k proxy_http_port -p -t int -- $PROXY_PORT
conn_settings -c -k proxy_https -p -t string -- $PROXY_HOST
conn_settings -c -k proxy_https_port -p -t int -- $PROXY_PORT
conn_settings -c -k proxytype -p -t string -- MANUAL

start_pkgmgrd.sh
set +e
s=`dbus_send install string:"hpm-user1" string:"" 2>&1`
match="^Error com.nokia.package_manager.Error.DownloadFailed: .*Could not connect to $PROXY_HOST:$PROXY_PORT"
echo $s|grep -q "$match"
if [ $? -ne 0 ]; then
    echo "FAIL: output '$s' does not match '$match'"
else
    echo "SUCCESS"
fi
