#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.

from xml.dom.minidom import parse, parseString

def find_package(doc, pkg):
    P = doc.getElementsByTagName('package')

    for p in P:
        def text(p, tag):
            return p.getElementsByTagName(tag)[0].firstChild.data
        if text(p, 'name') != pkg:
            continue

        return p

    return None

def delete_failed_install(filename, pkg):
    doc = parse(filename)
    p = find_package(doc, pkg)
    if p != None:
        doc.parentNode.removeChild(p)

def add_child_node(doc, parent, name, text):
    elem = doc.createElement(name)
    elem.appendChild(doc.createTextNode(text))
    parent.appendChild(elem)

def upsert_failed_install(filename, pkg, ver, dom=None, error_code=None, error_msg=None):
    """Insert or update failed install to filename."""
    try:
        doc = parse(filename)
    except:
        doc = parseString('<failed-installs/>')

    p = find_package(doc, pkg)
    if p != None:
        doc.firstChild.removeChild(p)

    data = doc.createElement('package')
    add_child_node(doc, data, 'name', pkg)
    add_child_node(doc, data, 'version', ver)
    if dom:
        add_child_node(doc, data, 'domain', dom)
    add_child_node(doc, data, 'error-code', error_code)
    add_child_node(doc, data, 'error-msg', error_msg)
    doc.firstChild.appendChild(data)
    doc.writexml(file(filename, 'w'))
