#!/usr/bin/python
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.

#
# Print test .deb generation makefile.

import string
from test_packages import old_packages, new_packages, old_other_packages, new_other_packages, old_trusted_repo_packages, new_trusted_repo_packages, old_non_trusted_repo_packages, new_non_trusted_repo_packages
from testconf import host_arch

def encode(s):
    return s.replace('\n', '\\n').replace('$', '$$')

def print_makefile(varname, P, key):
    global host_arch
    build_dir = 'tmp/%s' % host_arch

    for p in P:
        fields = []
        for (k, v) in p.fields.items():
            fields.append("-F %s='%s'" % (k, encode(v)))

        args = ['--build-dir=%s' % build_dir]
        if p.delays[0] > 0:
            args.append('--preinst-delay %d' % p.delays[0])
        if p.delays[1] > 0:
            args.append('--postinst-delay %d' % p.delays[1])
        if p.delays[2] > 0:
            args.append('--prerm-delay %d' % p.delays[2])
        if p.delays[3] > 0:
            args.append('--postrm-delay %d' % p.delays[3])

        files = []
        for f in p.files:
            files.append('-f %s' % f)

        if p.key:
            k = p.key
        else:
            k = key

        deb = "%s/%s" % (build_dir, p.deb_filename())
        print """%s: test_packages.py
	./make_deb.py %s %s %s %s %s
	./sign-deb -k ./testrepo-keys.gpg -s ./testrepo-secret-keys.gpg %s %s""" % (
            deb,
            ' '.join(fields),
            ' '.join(args),
            ' '.join(files),
            p.package, p.version,
            deb, k)

    print ""
    print "%s =" % varname,
    for p in P:
        print '%s/%s' % (build_dir, p.deb_filename()),
    print ""
    print ""

def main():
    global old_packages, new_packages
    global old_other_packages, new_other_packages
    global old_trusted_repo_packages, new_trusted_repo_packages
    global old_non_trusted_repo_packages, new_non_trusted_repo_packages

    print_makefile('old_packages', old_packages, '9E43D4ED')
    print_makefile('new_packages', new_packages, '9E43D4ED')
    print_makefile('old_other_packages', old_other_packages, '44423677')
    print_makefile('new_other_packages', new_other_packages, '44423677')
    print_makefile('old_trusted_packages', old_trusted_repo_packages, '9E43D4ED')
    print_makefile('new_trusted_packages', new_trusted_repo_packages, '9E43D4ED')
    print_makefile('old_non_trusted_packages', old_non_trusted_repo_packages, 'E0ECAC20')
    print_makefile('new_non_trusted_packages', new_non_trusted_repo_packages, 'E0ECAC20')
    
if __name__ == '__main__':
    main()
