#!/usr/bin/python
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.

#
# Create Debian packages for testing.

import os
import sys
import getopt
#sys.path.append(os.path.abspath('..'))

from test_helper import *

pkg_arch = deb_arch()
pkgdir = 'tmp'

def get_pkgdir():
    return '%s/%s' % (pkgdir, pkg_arch)

def num_pkgs():
    i = 0
    for p in pkgs:
        if 'versions' in p:
            i += len(p['versions'])
        else:
            i += 1
    return i

def fill_pkg(pkg):
    name = pkg['name']

    if 'arch' in pkg:
        arch = pkg['arch']
    else:
        arch = pkg_arch

    if 'versions' in pkg:
        vers = pkg['versions']
    else:
        vers = install_ver

    if 'comps' in pkg:
        comps = pkg['comps']
    else:
        comps = [user_comp]

    if 'deps' in pkg:
        deps = pkg['deps']
    else:
        deps = None

    if 'delays' in pkg:
        delays = pkg['delays']
    else:
        delays = [0, 0]

    if 'files' in pkg:
        files = pkg['files']
    else:
        files = False

    if 'binarysize' in pkg:
        binarysize = pkg['binarysize']
    else:
        binarysize = 0

    return name,arch,vers,comps,deps,delays,files,binarysize

def deb_filename(name, version=install_ver[0], arch=pkg_arch):
    return "%s_%s-1_%s.deb" % (name, version, arch)

def maybe_create_package(name, arch, version, comps, deps, delays, files, binarysize):
    deb = deb_filename(name, version, arch)

    if not os.path.isfile(deb):
        c = None
        if deps:
            c = "deb-create -d %s -n %s -V %s -s %s -a %s -i %d -I %d -S %d -D '%s' " % (get_pkgdir(), name, version, ' -s '.join(comps), arch, delays[0], delays[1], binarysize, deps)
        else:
            c = "deb-create -d %s -n %s -V %s -s %s -a %s -i %d -I %d -S %d" % (get_pkgdir(), name, version, ' -s '.join(comps), arch, delays[0], delays[1], binarysize)
        if files:
            c += ' -f'
        print "%s/%s:" % (get_pkgdir(), deb)
        print "\t./%s" % c
#        system_sbox(c)

def cmd_create_debs():
    i = 1
    n = num_pkgs()

    for pkg in pkgs:
        name,arch,vers,comps,deps,delays,files,binarysize = fill_pkg(pkg)

        for v in vers:
            maybe_create_package(pkg['name'], arch, v, comps, deps, delays, files, binarysize)
            i += 1

def cmd_print_debs():
    for pkg in pkgs:
        name,arch,vers,comps,deps,delays,files,binarysize = fill_pkg(pkg)

        for v in vers:
            print get_pkgdir()+'/'+deb_filename(name, v, arch)

def print_usage():
    print """Usage: %s [OPTION...] command

Commands:
  create      Create Debian test packages
  print       Print list of Debian packages to create

Options:
  -h, --help        Display this help message
  -d, --pkgdir=DIR  Use DIR as package creation directory
                    (default: %s)
""" % (sys.argv[0], pkgdir)

def main():
    global pkg_arch

    try:
        opts, args = getopt.getopt(sys.argv[1:], 'a:d:hv',
                                   ['arch=', 'pkgdir=', 'help', 'verbose'])
    except getopt.GetoptError, err:
        print str(err)
        print_usage()
        sys.exit(2)

    for o, a in opts:
        if o in ('-h', '--help'):
            print_usage()
            sys.exit(0)
        elif o in ('-v', '--verbose'):
            verbose = True
        elif o in ('-a', '--arch'):
            pkg_arch = a
        elif o in ('-d', '--pkgdir'):
            pkgdir = a

    if len(args) == 0:
        print_usage()
        os.exit(2)

    cmd = args[0]
    if cmd == 'create':
        cmd_create_debs()
    elif cmd == 'print':
        cmd_print_debs()

if __name__ == '__main__':
    main()
