#!/bin/bash
#
# Make a Debian repository and fill it with given Debian packages.
#
# TODO:
# - Command-line arguments

set -e

PATH=`dirname $0`:$PATH
NAME=testrepo
VERSION=1

usage() {
    echo "Usage: $0 [-n NAME] REPODIR DIST KEY DEB..." >/dev/stderr
    exit 2
}

while getopts n: opt; do
    case "$opt" in
	n) NAME="$OPTARG";;
	*) usage; exit2;;
    esac
done
shift `expr $OPTIND - 1`

if test -z "$4"; then
    usage
fi

ARCHIVE=$NAME.tar.gz

REPODIR=$1/$NAME
shift

DIST=$1
shift

KEY=$1
shift

if [ "$DIST" == "default" ]; then
  POOL=pool
else
  POOL=$DIST-pool
fi

for deb in "$@"; do
    if [ ! -f "$deb" ]; then
	echo "Error: file '$deb' not found. Stop" >/dev/stderr
	exit 1
    fi

    section=`dpkg --info $deb | grep "^ Section: " | awk '{ print $2 }'`
    if test -z `echo $section|grep '/'`; then
	section="main/$section"
    fi

    section_dir="$REPODIR/$POOL/$section"
    target=$section_dir/$(basename $deb)
    mkdir -p "$section_dir"
    cp "$deb" "$target"

done

deb-repo-init -d "$REPODIR" -n "$DIST" -s main -s user -k "$KEY" -p $POOL
