#!/bin/sh
#
# Install repository from given repository tarball.

set -e

SCRIPTDIR=`dirname $0`
#source $SCRIPTDIR/../testvars.sh
PATH=$PATH:`cd $SCRIPTDIR && pwd`
#REPOROOT='/var/tmp'

repo_archive=$1
repo_root=$2
dist=
sections=
repo_addr="http://127.0.0.1:8080/testrepo"
#repo_addr="file://$absdir"

usage() {
    echo "Usage: $0 repo-archive repo-rootdir" >/dev/stderr
    exit 2
}

read_repository_conf() {
    dist=`grep 'Tree ' $1|sed 's/.*[/]\(.*\)\".*$/\1/'`
    sections=`grep 'Sections ' $1|sed 's/.*\"\(.*\)\".*$/\1/'`
}

update_sources_list() {
    f=/etc/apt/sources.list.d/$1.list
    dir="$2"
    absdir=`cd $dir && pwd`
    test -f $f && rm $f
    cat >$f <<EOF
deb $repo_addr default $sections
deb $repo_addr other $sections
deb $repo_addr untrusted $sections
EOF
}

configure_domains() {
    if [ -f "$repo_dir/testrepo-origins.deb" ]; then
        AEGIS_FIXED_ORIGIN=com.nokia.maemo dpkg -i "$repo_dir/testrepo-origins.deb"
    fi
}

if test -z "$2"; then
    usage
fi

if test ! -f "$repo_archive"; then
    echo "Error: file '$repo_archive' not found. Stop." >/dev/stderr
    exit 1
fi

# When ran from testrunner.py, the following fails:
# `tar tzf $repo_archive|head -1|sed 's/[/]//'`
# tar: Error in writing to standard output
# tar: Error is not recoverable: exiting now

tmp=`mktemp tmp.XXXXXXXXXX`
tar tzf $repo_archive >$tmp
repo_name=`cat $tmp|head -1|sed 's/[/]//'`
rm $tmp
repo_dir="$repo_root/$repo_name"

# Uninstall first
#if test -f "$repo_dir/repository.conf"; then
#    repo-uninstall.sh "$repo_dir"
#fi

#if test -d "$repo_dir"; then
#    echo "Error: directory '$repo_dir' already exist and it does not contain file repository.conf. Stop." >/dev/stderr
#    exit 1
#fi

mkdir -p "$repo_root"

tar xmzf $repo_archive -C $repo_root

repo_conf="$repo_dir/repository.conf"
if test ! -f $repo_conf; then
    echo "Error: file '$repo_conf' not found. Stop." >/dev/stderr
    exit 1
fi
read_repository_conf $repo_conf

if test -z "$dist" || test -z "$sections"; then
    echo "Error: could not resolve dist or sections from $repo_conf. Stop." >/dev/stderr
    exit 2
fi

update_sources_list $repo_name $repo_dir

configure_domains $repo_dir

start-webserver.sh
