#!/bin/sh
#
# Uninstall (remove) given repository directory.

set -e
source `dirname $0`/testvars.sh

if [ ! -x $SCRIPTDIR/../stop_pkgmgrd.sh ]; then
    PATH=$PATH:/usr/share/package-manager-tests
fi

apt_get() {
    no_proxy=127.0.0.1 $FAKEROOT apt-get.real $@ </dev/null
}

usage() {
    echo "Usage: $0 repo-directory" >/dev/stderr
    exit 2
}

package_name() {
    deb=$1

    echo `echo $deb|sed 's/.*[/]\([^_]*\)_.*$/\1/'`
}

uninstall_test_debs() {
    set +e
    pkgs=`dpkg --get-selections|sed 's/^\(.*\)\t.*/\1/'|grep '^hpm-'`

    apt_get -qq remove $pkgs
    dpkg -r local-package

    set -e
}

if test -z "$1"; then
    usage
fi

dir="$1"

if test -d "$dir"; then
    if test ! -f "$dir/repository.conf"; then
	echo "Error: File '$dir/repository.conf' not found, refusing to continue. Stop." >/dev/stderr
	exit 1
    fi

    stop-pkgmgrd

    uninstall_test_debs $dir
    test -d $dir && rm -rf $dir

    stop-webserver.sh

    repo_name=`echo $dir|sed 's/.*[/]//'`
    sources_file="/etc/apt/sources.list.d/$repo_name.list"
    test -f $sources_file && rm $sources_file

    rm -f /etc/apt/trusted.gpg.d/testrepo-keys.gpg
    rm -f /usr/share/package-manager/domains/testrepo-domains.xexp
    dpkg -r testrepo-origins

    set +e
    apt_get -qq update
fi
