# Package Manager test packages
# -*- coding: utf-8 -*-
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.

from testconf import host_arch
import commands
import os

v0 = '0.1.0-1'
v1 = '0.1.1-1'
v2 = '0.1.2-1'

# Default value.
create_nft_packages = False

class Package:
    def __init__(self, package, v=v0, delays=[], fields={}, files=[], key=None):
        global host_arch
        self.package = package
        self.version = v
        self.delays = delays
        self.fields = fields
        # files[0] == local filename, files[1] == package filename
        self.files = files
        self.key = key

        # Set defaults of missing fields:
        self._set_nil_field('Architecture', host_arch)
        self._set_nil_field('Section', 'user/development')
        self._set_nil_field('Installed-Size', "16")
        self._set_nil_field('Maintainer', 'Maintainer name <package@maintainer>')
        self._set_nil_field('Description', "%s package\n Long description for package %s" % (self.package, self.package))
        self._set_nil_field('Maemo-Display-Name', 'Fancy %s' % self.package)
        self._set_nil_field('Maemo-Upgrade-Description', '%s upgrade description' % self.package)
        for lang in ('de_DE', 'fi_FI', 'sv_SE'):
            self._set_nil_field('Description-%s' % lang, "%s package description (%s)\n Long description for package %s (%s)" % (self.package, lang, self.package, lang))
            self._set_nil_field('Maemo-Display-Name-%s' % lang, '%s (%s)' % (self.get_field('Maemo-Display-Name'), lang))
            self._set_nil_field('Maemo-Upgrade-Description-%s' % lang, '%s upgrade description (%s)' % (self.package, lang))

        self._set_nil_field('Maemo-Required-Free-Space', self.fields['Installed-Size'])

    def _set_nil_field(self, name, value):
        if not name in self.fields:
            self.fields[name] = value

    def get_field(self, name):
        return self.fields[name]

    def deb_filename(self):
        return '%s_%s_%s.deb' % (self.package, self.version, self.fields['Architecture'])

def read_icon_data(filename):
    d = os.path.dirname(__file__)
    return commands.getstatusoutput('%s/base64encode.sh %s/%s' % (d, d, filename))

def generate_data(size, outputfile):
    os.system('%s/datagenerator.sh %s %s/%s' % (os.path.dirname(__file__), size, os.path.dirname(__file__), outputfile))

# Array of Package objects with version=='0.1.0-1'
old_packages = []
# Array of Package objects with version=='0.1.1-1' or higher
new_packages = []

invalid_arch = 'powerpc'
user_section = 'user/development'
nonuser_section = 'misc'
user_sections = (
    'user/desktop',
    'user/development',
    'user/education',
    'user/games',
    'user/graphics',
    'user/multimedia',
    'user/navigation',
    'user/network',
    'user/office',
    'user/science',
#    'user/system',
    'user/utilities'
    )

def add(P, package, v=v0, delays=[0, 0, 0, 0], fields={}, files=[], key=None):
    global host_arch
    P.append(Package(package, v, delays, fields.copy(), files, key))

deb_build_opt = os.getenv('DEB_BUILD_OPTIONS')
if deb_build_opt != None and 'nft' in deb_build_opt:
    create_nft_packages = True

P = []
add(P, 'hpm-user-invalid-arch', fields={'Architecture': invalid_arch})
add(P, 'hpm-user-missing-deps', fields={'Depends': 'hpm-nonexisting'})
add(P, 'hpm-nonuser', fields={'Section': nonuser_section})
add(P, 'hpm-user-failed-dep', v=v0, fields={'Depends': 'hpm-user-preinst-fail'})
add(P, 'hpm-user-failed-dep', v=v1 )
add(P, 'hpm-user-dep-on-postinst-fail', v=v0, fields={'Depends': 'hpm-user-postinst-fail'})
add(P, 'hpm-user-indirect-dep-on-postinst-fail', v=v0, fields={'Depends': 'hpm-user-dep-on-postinst-fail'})
for ver in [v0, v1]:
    add(P, 'hpm-user-system', v=ver, fields={'Section': 'user/system'})

    add(P, 'hpm-nonuser-dep', v=ver, fields={'Section': nonuser_section})
    add(P, 'hpm-nonuser-dep2', v=ver, fields={'Section': nonuser_section})
    add(P, 'hpm-user-valid-deps', v=ver, fields={'Depends': 'hpm-user-dep1, hpm-user-dep2, hpm-user-dep3, hpm-user-dep4, hpm-user-dep5, hpm-nonuser-dep, hpm-nonuser-dep2'})
    add(P, 'hpm-user-valid-deps2', v=ver, fields={'Depends': 'hpm-user-dep1, hpm-user-dep2, hpm-user-dep3, hpm-user-dep4, hpm-user-dep5, hpm-nonuser-dep, hpm-nonuser-dep2'})
    add(P, 'hpm-user-invalid-dep', v=ver, fields={'Depends': 'hpm-invalid-dep'})

# Conflicts/Breaks packages:
add(P, 'hpm-user-conflicts-hpm-user-old-deps', v=v0, fields={'Conflicts': 'hpm-user-old-deps'})
add(P, 'hpm-user-breaks-hpm-user-old-deps', v=v0, fields={'Breaks': 'hpm-user-old-deps', 'Replaces': 'hpm-user-old-deps'})
add(P, 'hpm-user-upgrade-conflicts-hpm-user-old-deps', v=v0)
add(P, 'hpm-user-upgrade-conflicts-hpm-user-old-deps', v=v1, fields={'Conflicts': 'hpm-user-old-deps'})
add(P, 'hpm-user-upgrade-breaks-hpm-user-old-deps', v=v0)
add(P, 'hpm-user-upgrade-breaks-hpm-user-old-deps', v=v1, fields={'Breaks': 'hpm-user-old-deps', 'Replaces': 'hpm-user-old-deps'})

add(P, 'hpm-user-invalid-dep2', v=v0)
add(P, 'hpm-user-invalid-dep2', v=v1, fields={'Depends': 'hpm-invalid-dep'})

add(P, 'hpm-user-old-deps', v=v0, fields={'Depends': 'hpm-user-dep2 (= %s)' % v0})
add(P, 'hpm-user-valid-deps3', v=v0, fields={'Depends': 'hpm-user-dep1, hpm-user-dep2, hpm-user-dep3, hpm-user-dep6'})
add(P, 'hpm-user-valid-deps3', v=v1, fields={'Depends': 'hpm-user-dep1, hpm-user-dep2 (= 0.1.1-1), hpm-user-dep3, hpm-user-dep4, hpm-user-dep5 (= 0.1.1-1)', 'Conflicts' : 'hpm-user-dep6, hpm-user-old-deps', 'Replaces' : 'hpm-user-dep6, hpm-user-old-deps'})
add(P, 'hpm-user-valid-deps4', v=v0, fields={'Depends': 'hpm-user-dep1, hpm-user-dep2, hpm-user-dep3, hpm-user-dep6'})
add(P, 'hpm-user-valid-deps4', v=v1, fields={'Depends': 'hpm-user-dep1, hpm-user-dep2 (= 0.1.1-1), hpm-user-dep3, hpm-user-dep4, hpm-user-dep5 (= 0.1.1-1)', 'Conflicts' : 'hpm-user-dep6', 'Replaces' : 'hpm-user-dep6'})

# Downgrade packages:
add(P, 'hpm-user-upgrade-downgrades-hpm-user-dep1', v=v0, fields={'Depends': 'hpm-user-dep1 (>= %s)' % v1})
add(P, 'hpm-user-upgrade-downgrades-hpm-user-dep1', v=v1, fields={'Depends': 'hpm-user-dep1 (<= %s)' % v0})

add(P, 'hpm-user-upgrade-old-deps', v=v0, fields={'Depends': 'hpm-user-dep2'})
add(P, 'hpm-user-upgrade-old-deps', v=v1, fields={'Depends': 'hpm-user-dep2 (%s)' % v0})

add(P, 'hpm-user-huge-install', fields={'Install-Size': '5432109876543210', 'Maemo-Required-Free-Space': '5432109876543210'})
add(P, 'hpm-user-files', files=['../../README=/usr/share/hpm-user-files/README'])

if create_nft_packages:
    for i in range(1, 11):
        generate_data('%d00' % i, 'test_%d00_kb_data.txt' % i)
        add(P, 'hpm-user-1000-%d' % i, fields={'Section' : 'user/hidden'}, files=['test_%d00_kb_data.txt=/usr/share/hpm-user-1000-%d-files/test_%d00_kb_data.txt' % (i,i,i)])

    # add package containing many same size of test-files: 50 * 100 kB
    testfiles=[]
    for i in range(1, 50):
        # assuming that test_100_kb_data.txt is already generated
        testfiles.append('test_100_kb_data.txt=/usr/share/hpm-user-2000-1-files/test_100_kb_%s_data.txt' % i)
    add(P, 'hpm-user-2000-1', fields={'Section' : 'user/hidden'}, files=testfiles)

    # add package containing many same size of test-files: 50 * 1000 kB
    testfiles=[]
    for i in range(1, 50):
        # assuming that test_1000_kb_data.txt is already generated
        testfiles.append('test_1000_kb_data.txt=/usr/share/hpm-user-2000-2-files/test_1000_kb_%s_data.txt' % i)
    add(P, 'hpm-user-2000-2', fields={'Section' : 'user/hidden'}, files=testfiles)

    # add different size of files: 100 kb ... 10000 kB
    testfiles=[]
    for i in range(1, 9):
        testfiles.append('test_100_kb_data.txt=/usr/share/hpm-user-2000-3-files/test_100_kb_%s_data.txt' % i)
        testfiles.append('test_1000_kb_data.txt=/usr/share/hpm-user-2000-3-files/test_1000_kb_%s_data.txt' % i)
    generate_data('10000', 'test_10000_kb_data.txt')
    for i in range(1, 5):
        testfiles.append('test_10000_kb_data.txt=/usr/share/hpm-user-2000-3-files/test_10000_kb_%s_data.txt' % i)
    add(P, 'hpm-user-2000-3', fields={'Section' : 'user/hidden'}, files=testfiles)

# Updated
add(P, 'hpm-user-upgrade-broken', v=v0, 
	fields={'Section' : 'user/development', 
	'MeeGo-Desktop-Entry': '[Desktop Entry]\n Type=Application\n Name=hpm-user-upgrade-broken\n Icon=icon-m-content-debian',
	'MeeGo-Desktop-Entry-Filename' : 'hpm-user-upgrade-broken.desktop'},
    files=['hpm-user-upgrade-broken.desktop=/usr/share/applications/hpm-user-upgrade-broken.desktop'])
add(P, 'hpm-user-upgrade-broken', v=v1, 
	fields={'Section' : 'user/development', 
	'MeeGo-Desktop-Entry': '[Desktop Entry]\n Type=Application\n Name=hpm-user-upgrade-broken\n Icon=icon-m-content-debian',
	'MeeGo-Desktop-Entry-Filename' : 'hpm-user-upgrade-broken.desktop'},
    files=['hpm-user-upgrade-broken.desktop=/usr/share/applications/hpm-user-upgrade-broken.desktop'])
add(P, 'hpm-user-upgrade-broken', v=v2, 
	fields={'Depends' : 'hpm-user-postinst-fail', 'Section' : 'user/development', 
	'MeeGo-Desktop-Entry': '[Desktop Entry]\n Type=Application\n Name=hpm-user-upgrade-broken\n Icon=icon-m-content-debian',
	'MeeGo-Desktop-Entry-Filename' : 'hpm-user-upgrade-broken.desktop'},
    files=['hpm-user-upgrade-broken.desktop=/usr/share/applications/hpm-user-upgrade-broken.desktop'])
add(P, 'hpm-user-depends-user1', v=v0, fields={'Depends': 'hpm-user1 (= %s)' % v0})
add(P, 'hpm-user-depends-user1', v=v1, fields={'Depends': 'hpm-user1 (= %s)' % v2})

add(P, 'hpm-user-depends-user-deps', v=v0, fields={'Depends': 'hpm-user-dep1, hpm-user-dep2'})
add(P, 'hpm-user-depends-user-deps', v=v1, fields={'Depends': 'hpm-user-dep1, hpm-user-dep2, hpm-user-dep3'})

add(P, 'hpm-user-upgrade-fail', v=v0)
add(P, 'hpm-user-upgrade-fail', v=v1, fields={'Depends': 'hpm-user-preinst-fail'})
add(P, 'hpm-user-upgrade-fail', v=v2)

add(P, 'hpm-user-fail', v=v0, fields={'Depends': 'hpm-nonuser-fail'})
add(P, 'hpm-user-fail', v=v1, fields={'Depends': 'hpm-nonuser-fail'})
add(P, 'hpm-user-fail', v=v2, fields={'Depends': 'hpm-nonuser-fail'})

# User package which have update version available already in testrepo1
add(P, 'hpm-user-upgrade-available', v='0.0.0-1')
add(P, 'hpm-user-upgrade-available', v=v0)
add(P, 'hpm-user-upgrade-available', v=v1)

# User package which depends an old system package
add(P, 'hpm-user-depends-old-os', v=v0, fields={'Depends': 'hpm-system1 (= %s)' % v0})
add(P, 'hpm-user-depends-old-os', v=v1, fields={'Depends': 'hpm-system1 (= %s)' % v0})

i = 0
for s in user_sections:
    i += 1
    for ver in [v0, v1]:
        add(P, 'hpm-user%d' % i, v=ver, fields={'Section': s})

add(P, 'hpm-user-visible', v=v0, fields={'Section': nonuser_section, 'Maemo-Flags': 'visible'})

for i in range(1, 11):
    for ver in [v0, v1]:
        add(P, 'hpm-user-dep%d' % i, v=ver)

if create_nft_packages:
    # Generates 100 packages with 1MB file
    generate_data('1027', 'test_0_data.txt')
    for i in range(1, 101):
        for ver in [v0, v1]:
            add(P, 'hpm-user-100-%d' % i, v=ver, fields={'Section': 'user/hidden'}, files=['test_0_data.txt=/usr/share/hpm-user-100-%d-files/test_0_data.txt' % i])

add(P, 'hpm-user-upgrade-no-upgrade')

for ver in [v0, v1, v2]:
    add(P, 'hpm-long-install', delays=[20, 20, 10, 10], v=ver)

i = 0
for ver in [v0, v1]:
    for i in range(1, 3):
        icon_data = read_icon_data('%d.png' % i)[1]
        add(P, 'hpm-user-upgrade%d' % i, v=ver, fields={'Maemo-Icon-26': icon_data})
    add(P, 'hpm-user-upgrade-dep', v=ver)
    for i in range(1, 6):
        add(P, 'hpm-user-upgrade-dep%d' % i, v=ver)
    add(P, 'hpm-nonuser-upgrade', v=ver, fields={'Section': nonuser_section})
    add(P, 'hpm-nonuser-upgrade-dep', v=ver, fields={'Section': nonuser_section})
    add(P, 'hpm-user-upgrade-valid-deps', v=ver, fields={'Depends': 'hpm-user-upgrade-dep1, hpm-user-upgrade-dep2, hpm-user-upgrade-dep3, hpm-user-upgrade-dep4, hpm-user-upgrade-dep5, hpm-nonuser-dep'})

add(P, 'hpm-user-upgrade-missing-deps', v=v0, fields={'Depends': 'hpm-user-upgrade-dep, hpm-nonuser-upgrade-dep'})
add(P, 'hpm-user-upgrade-missing-deps', v=v1, fields={'Depends': 'hpm-user-upgrade-dep, hpm-nonuser-upgrade-dep, hpm-nonexisting'})

add(P, 'hpm-user-upgrade-nonuser', v=v0)
add(P, 'hpm-user-upgrade-nonuser', v=v1)
add(P, 'hpm-user-upgrade-nonuser', v=v2, fields={'Section': nonuser_section})

# System packages
j = 0
for ver in [v0, v1, v2]:
    j += 1
    icon_data = read_icon_data('%d.png' % i)[1]

    D = []
    for i in range(1, 4):
        p = 'hpm-system%d' % i
        add(P, p, v=ver, fields={'Section': nonuser_section, 'Maemo-Flags': 'reboot'})
        D.append('%s (= %s)' % (p, ver))

    add(P, 'hpm-system4', v=ver, fields={'Section': nonuser_section, 'Priority': 'optional'})

    system_reboot_update_packages = []

    for i in range(1, 4):
        p = 'hpm-system-reboot-update%d' % i
        add(P, p, v=ver, fields={'Section': nonuser_section, 'Maemo-Flags': 'reboot-update'})
        system_reboot_update_packages.append('%s (= %s)' % (p, ver))

    add(P, 'hpm-system-meta', v=ver, fields={'Priority': 'required', 'Maemo-Flags': 'system-update', 'Depends': ', '.join(D), 'Section': 'user/system', 'Maemo-Icon-26': icon_data})
    add(P, 'hpm-system-meta-reboot-update', v=ver, fields={'Priority': 'required', 'Depends': ', '.join(D), 'Section': 'user/system', 'Maemo-Icon-26': icon_data, 'Maemo-Flags': 'system-update, reboot-update'})

    add(P, 'hpm-system-meta-reboot-update-from-dependency', v=ver, fields={'Priority': 'required', 'Depends': ', '.join(system_reboot_update_packages), 'Section': 'user/system', 'Maemo-Icon-26': icon_data, 'Maemo-Flags': 'system-update'})

    add(P, 'hpm-system-meta-reboot', v=ver, fields={'Priority': 'required', 'Depends': ', '.join(D[1:]), 'Section': 'user/system', 'Maemo-Flags': 'system-update, reboot'})
    add(P, 'hpm-system-meta-noreboot', v=ver, fields={'Priority': 'required', 'Maemo-Flags': 'system-update', 'Section': 'user/system'})

    if create_nft_packages:
        generate_data('15333', 'test_0_data_15.txt')
        add(P, 'hpm-system-meta-15-reboot', v=ver, fields={'Priority': 'required', 'Depends': ', '.join(D), 'Section': 'user/hidden', 'Maemo-Flags': 'system-update, reboot'}, files=['test_0_data_15.txt=/usr/share/hpm-system-meta-15-noreboot-files/test_0_data_15.txt'])
        add(P, 'hpm-system-meta-15-noreboot', v=ver, fields={'Priority': 'required', 'Maemo-Flags': 'system-update', 'Section': 'user/hidden'}, files=['test_0_data_15.txt=/usr/share/hpm-system-meta-15-noreboot-files/test_0_data_15.txt'])

    add(P, 'hpm-system-meta-long-noreboot', v=ver, fields={'Priority': 'required', 'Depends': 'hpm-long-install (= %s)' % (ver), 'Maemo-Flags': 'system-update', 'Section': 'user/system'})

    D.append('%s (= %s)' % ('hpm-long-install', ver))
    add(P, 'hpm-system-meta-long', v=ver, fields={'Priority': 'required', 'Depends': ', '.join(D), 'Section': 'user/system', 'Maemo-Flags': 'system-update, reboot'})

    D.append('%s (= %s)' % ('hpm-user-dep1', v0))
    add(P, 'hpm-system-meta-depends-old-package', v=ver, fields={'Priority': 'required', 'Depends': ', '.join(D), 'Section': 'user/system', 'Maemo-Flags': 'system-update'})


# System metapackage whose upgrade fails but rescue succeeds
add(P, 'hpm-system-meta-fails-once', v=v0, fields={'Priority': 'required', 'Maemo-Flags': 'system-update', 'Section': 'user/system', 'Maemo-Icon-26': icon_data})
add(P, 'hpm-system-meta-fails-once', v=v1, fields={'Priority': 'required', 'Maemo-Flags': 'system-update', 'Depends': 'hpm-user-postinst-fail-once', 'Section': 'user/system', 'Maemo-Icon-26': icon_data})

# System metapackage that depends / update / remove user packages
add(P, 'hpm-system-meta-depends-user-packages', v=v0, fields={'Maemo-Flags': 'system-update',
                                  'Section': 'user/system',
                                  'Depends': 'hpm-user-dep1, hpm-user-dep2, hpm-user-dep3, hpm-user-dep6'})
add(P, 'hpm-system-meta-depends-user-packages', v=v1, fields={'Maemo-Flags': 'system-update',
                                  'Section': 'user/system',
                                  'Depends': 'hpm-user-dep1, hpm-user-dep2 (=%s), hpm-user-dep3, hpm-user-dep4' % v1,
                                  'Conflicts': 'hpm-user-dep6, hpm-user-old-deps (=%s)' % v0 })

# System metapackage update downgrades user packages
add(P, 'hpm-system-meta-downgrades-user-packages', v=v0, fields={'Maemo-Flags': 'system-update',
                                  'Section': 'user/system',
                                  'Depends': 'hpm-user-upgrade-available (=%s)' % v0 })
add(P, 'hpm-system-meta-downgrades-user-packages', v=v1, fields={'Maemo-Flags': 'system-update',
                                  'Section': 'user/system',
                                  'Depends': 'hpm-user-upgrade-available (=0.0.0-1)' })


# System metapackage that fails to upgrade due to preinst fail in dep
add(P, 'hpm-system-meta-fail', v=v0, fields={'Section': 'user/system', 'Priority': 'required', 'Maemo-Flags': 'system-update'})
add(P, 'hpm-system-meta-fail', v=v1, fields={'Section': 'user/system', 'Priority': 'required', 'Maemo-Flags': 'system-update', 'Depends': 'hpm-user-preinst-fail (= 0.1.0-1)'})

# System metapackage that fails to upgrade due to missing file in testrepo
add(P, 'hpm-system-meta-fail-download', v=v0, fields={'Section': 'user/system', 'Priority': 'required', 'Maemo-Flags': 'system-update'})
add(P, 'hpm-system-meta-fail-download', v=v1, fields={'Section': 'user/system', 'Priority': 'required', 'Maemo-Flags': 'system-update'})

add(P, 'hpm-system-meta-missing-dep', v=v0,  fields={'Section': 'user/system', 'Priority': 'required', 'Maemo-Flags': 'system-update'})
add(P, 'hpm-system-meta-missing-dep', v=v1,  fields={'Section': 'user/system', 'Priority': 'required', 'Maemo-Flags': 'system-update', 'Depends': 'hpm-nonexisting'})

# System update package that has missing system-update flag (PkMgr-1399)

add(P, 'hpm-system-meta-missing-update-flag', v=v0,  fields={'Section': 'user/system', 'Priority': 'required', 'Maemo-Flags': 'system-update'})
add(P, 'hpm-system-meta-missing-update-flag', v=v1,  fields={'Section': 'user/system', 'Priority': 'required'})

add(P, 'hpm-system-meta-reckless', v=v0,  fields={'Section': 'user/system', 'Maemo-Flags': 'system-update'})
add(P, 'hpm-system-meta-reckless', v=v1,  fields={'Section': 'user/system', 'Maemo-Flags': 'system-update'})

add(P, 'hpm-system-meta-no-reckless', v=v0,  fields={'Section': 'user/system', 'Maemo-Flags': 'system-update'})
add(P, 'hpm-system-meta-no-reckless', v=v1,  fields={'Section': 'user/system', 'Maemo-Flags': 'system-update, no-reckless'})

# User packages update claims to be OS metapackage
add(P, 'hpm-upgrade-is-os-package', v=v0, fields={'Section': 'user/development'})
add(P, 'hpm-upgrade-is-os-package', v=v1, fields={'Section': 'user/development', 'Priority': 'required', 'Maemo-Flags': 'system-update'})

add(P, 'hpm-fake-system-pkg', v=v0, fields={'Maemo-Flags': 'system-update'})

add(P, 'hpm-fake-system-upgrade', v=v0)
add(P, 'hpm-fake-system-upgrade', v=v1, fields={'Maemo-Flags': 'system-update'})

# System update with special characters in details
desc = 'This is a åäö OS update'
add(P, 'hpm-system-meta-special', v=v0,  fields={'Maemo-Upgrade-Description': desc, 'Section': 'user/system', 'Maemo-Flags': 'system-update'})
add(P, 'hpm-system-meta-special', v=v1,  fields={'Maemo-Upgrade-Description': desc, 'Section': 'user/system', 'Maemo-Flags': 'system-update'})

# # Broken system package (requires package not on server)
# D = []
# for i in range(1, 4):
#     p = 'hpm-system%d' % i
# add(P, 'hpm-system-meta', v='0.1.2-1', fields={'Depends': ', '.join(D), 'Section': 'user/system'})

add(P, 'hpm-user-depends-fake-system-pkg', v=v0, fields={'Depends': 'hpm-fake-system-pkg (<= %s)' % v0})
add(P, 'hpm-user-upgrade-depends-fake-system-pkg', v=v0)
add(P, 'hpm-user-upgrade-depends-fake-system-pkg', v=v1, fields={'Depends': 'hpm-fake-system-pkg (<= %s)' % v0})

add(P, 'hpm-user-depends-old-system-meta', v=v0, fields={'Depends': 'hpm-system-meta-noreboot (<= %s)' % v0})
add(P, 'hpm-user-conflicts-system-meta', v=v0, fields={'Conflicts': 'hpm-system-meta-noreboot (= %s)' % v1})
add(P, 'hpm-user-breaks-system-meta', v=v0, fields={'Breaks': 'hpm-system-meta-noreboot', 'Replaces': 'hpm-system-meta-noreboot'})

add(P, 'hpm-user-depends-system-upgrade', v=v0)
add(P, 'hpm-user-depends-system-upgrade', v=v1, fields={'Depends': 'hpm-system-meta (= %s)' % v2})

add(P, 'hpm-user-depends-system1', v=v0, fields={'Depends': 'hpm-system1 (= %s)' % v0})
add(P, 'hpm-user-depends-system1', v=v1, fields={'Depends': 'hpm-system1 (= %s)' % v2})

add(P, 'hpm-user-upgrade-depends-nonuser-dep', v=v0 )
add(P, 'hpm-user-upgrade-depends-nonuser-dep', v=v1, fields={'Depends': 'hpm-nonuser-dep'})

add(P, 'hpm-user-upgrade-depends-nonuser-dep2', v=v0 )
add(P, 'hpm-user-upgrade-depends-nonuser-dep2', v=v1, fields={'Depends': 'hpm-nonuser-dep (= %s)' % v1 })

add(P, 'hpm-user-depends-new-package', v=v0, fields={'Depends': 'hpm-user-new-package'})
add(P, 'hpm-user-new-package', v=v1)

add(P, 'hpm-user-application', v=v0,
    fields={'MeeGo-Desktop-Entry': '[Desktop Entry]\n Type=Application\n Name=hpm-user-application\n Icon=icon-m-content-debian',
            'MeeGo-Desktop-Entry-Filename' : 'hpm-user-application.desktop'},
    files=['hpm-user-application.desktop=/usr/share/applications/hpm-user-application.desktop'])

add(P, 'hpm-user-application', v=v1,
    fields={'MeeGo-Desktop-Entry': '[Desktop Entry]\n Type=Application\n Name=hpm-user-application\n Icon=icon-m-content-debian',
            'MeeGo-Desktop-Entry-Filename' : 'hpm-user-application.desktop'},
    files=['hpm-user-application.desktop=/usr/share/applications/hpm-user-application.desktop'])

add(P, 'hpm-user-application-fail', v=v0, 
    fields={'MeeGo-Desktop-Entry': '[Desktop Entry]\n Type=Application\n Name=hpm-user-application-fail\n Icon=icon-m-content-debian', 'MeeGo-Desktop-Entry-Filename' : 'hpm-user-application-fail.desktop', 'Depends': 'hpm-user-postinst-fail'},
    files=['hpm-user-application-fail.desktop=/usr/share/applications/hpm-user-application-fail.desktop'])

add(P, 'hpm-user-application-upgrade-fail', v=v0,
    fields={'MeeGo-Desktop-Entry': '[Desktop Entry]\n Type=Application\n Name=hpm-user-application-upgrade-fail\n Icon=icon-m-content-debian',
            'MeeGo-Desktop-Entry-Filename' : 'hpm-user-application-upgrade-fail.desktop'},
    files=['hpm-user-application-upgrade-fail.desktop=/usr/share/applications/hpm-user-application-upgrade-fail.desktop'])

add(P, 'hpm-user-application-upgrade-fail', v=v1, 
    fields={'MeeGo-Desktop-Entry': '[Desktop Entry]\n Type=Application\n Name=hpm-user-application-upgrade-fail\n Icon=icon-m-content-debian', 'MeeGo-Desktop-Entry-Filename' : 'hpm-user-application-upgrade-fail.desktop', 'Depends': 'hpm-user-postinst-fail'},
    files=['hpm-user-application-upgrade-fail.desktop=/usr/share/applications/hpm-user-application-upgrade-fail.desktop'])

add(P, 'hpm-user-application-depends-old-system-meta-noreboot', v=v0,
    fields={'MeeGo-Desktop-Entry': '[Desktop Entry]\n Type=Application\n Name=hpm-user-application-depends-old-system-meta-noreboot\n Icon=icon-m-content-debian',
            'MeeGo-Desktop-Entry-Filename' : 'hpm-user-application-depends-old-system-meta-noreboot.desktop',
            'Depends': 'hpm-system-meta-noreboot (= %s)' % v0 },
    files=['hpm-user-application-depends-old-system-meta-noreboot.desktop=/usr/share/applications/hpm-user-application-depends-old-system-meta-noreboot.desktop'])


desc = '<b>No upgrade description</b>\n <p>This package has no upgrade description.</p>Versions available:<br><table border="1"><tr><th>Version</th><th>Description</th></tr><tr><td>0.1.0-1</td><td>First version</td></tr><tr><td>0.1.1-1</td><td>Revised version with same description</td></tr></table>'
add(P, 'hpm-user-no-upgrade-description', v=v0, fields={'Description': desc, 'Maemo-Upgrade-Description': ''})
add(P, 'hpm-user-no-upgrade-description', v=v1, fields={'Description': desc, 'Maemo-Upgrade-Description': ''})

# HJa: Related test case: OS update - user package depending on non-latest OS update / ID: 11356
add(P, 'hpm-user-application-depends-one-system-meta-noreboot', v=v0,
    fields={'MeeGo-Desktop-Entry': '[Desktop Entry]\n Type=Application\n Name=hpm-user-application-depends-one-system-meta-noreboot\n Icon=icon-m-content-debian',
            'MeeGo-Desktop-Entry-Filename' : 'hpm-user-application-depends-one-system-meta-noreboot.desktop',
            'Depends': 'hpm-system-meta-noreboot (= %s)' % v1 },
    files=['hpm-user-application-depends-one-system-meta-noreboot.desktop=/usr/share/applications/hpm-user-application-depends-one-system-meta-noreboot.desktop'])

# Error test packages
for v in (v0, v1):
    for p in ('RemovedBySystemUpgrade', 'OperationFailed', 'NotEnoughSpace', 'LowBatteryLevel', 'UpgradeFailed', 'DownloadFailed', 'Internal'):
        add(P, 'hpm-user-app-%s' % p.lower(), v=v,
            fields={'Maemo-Display-Name': p,
                    'MeeGo-Desktop-Entry': '[Desktop Entry]\n Type=Application\n Name=hpm-user-app-%s\n Icon=icon-m-content-debian' % p,
                    'MeeGo-Desktop-Entry-Filename' : 'hpm-user-app-%s.desktop' % p.lower()},
            files=['hpm-user-app-%s.desktop=/usr/share/applications/hpm-user-app-%s.desktop' % (p.lower(), p.lower())])

# Package with changing details
icon_1 = read_icon_data('1.png')[1]
icon_2 = read_icon_data('2.png')[1]
add(P, 'hpm-user-updating-dep', v=v0, fields={'Maemo-Icon-26': icon_1})
add(P, 'hpm-user-updating-dep', v=v1, fields={'Maemo-Icon-26': icon_2, 'Maemo-Display-Name': 'New Fancy hpm-user-updating-dep'})

add(P, 'hpm-user-updates-dep', v=v0, fields={'Depends': 'hpm-user-updating-dep (= %s)' % v0 })
add(P, 'hpm-user-updates-dep', v=v1, fields={'Depends': 'hpm-user-updating-dep (= %s)' % v1, 'Maemo-Display-Name': 'New Fancy hpm-user-updates-dep' })

add(P, 'hpm-user-corrupted-upgrade', v=v0)
add(P, 'hpm-user-corrupted-upgrade', v=v1, fields={'Description': 'Failed checksum'})
add(P, 'hpm-user-corrupted-upgrade', v=v2, fields={'Description': 'Incorrect size'})

# Updating package brokes the installation
add(P, 'hpm-user-upgrade-broken-deps', v=v0, fields={'Depends': 'hpm-user-corrupted-upgrade (= %s)' % v0 })
add(P, 'hpm-user-upgrade-broken-deps', v=v1, fields={'Depends': 'hpm-user-corrupted-upgrade (= %s)' % v1 })
add(P, 'hpm-user-upgrade-broken-deps', v=v2, fields={'Depends': 'hpm-user-corrupted-upgrade (= %s)' % v2 })

add(P, 'hpm-user-huge-update', v=v0 )
add(P, 'hpm-user-huge-update', v=v1, fields={'Install-Size': '5432109876543210', 'Maemo-Required-Free-Space': '5432109876543210'})

add(P, 'hpm-user-huge-deps', v=v0, fields={'Depends': 'hpm-user-huge-install' })

# Test package with longer description
desc = 'A long synopsis line of hpm-user-test-metadata to test the displaying it in the user interface\n Lorem ipsum\n .\n Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do\n eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut\n enim ad minim veniam, quis nostrud exercitation ullamco laboris\n nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in\n reprehenderit in voluptate velit esse cillum dolore eu fugiat\n nulla pariatur. Excepteur sint occaecat cupidatat non proident,\n sunt in culpa qui officia deserunt mollit anim id est laborum.\n  How about some bullets:\n  * Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed\n  * do eiusmod tempor incididunt ut labore et dolore magna\n  * aliqua. Ut enim ad minim veniam, quis nostrud exercitation\n  * ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis\n  * aute irure dolor in reprehenderit in voluptate velit esse\n  * cillum dolore eu fugiat nulla pariatur. Excepteur sint\n  * occaecat cupidatat non proident, sunt in culpa qui officia\n  * deserunt mollit anim id est laborum.\n .\n  Homepage: <URL:http://www.nokia.com/>'
add(P, 'hpm-user-test-metadata', v=v0, fields={'Description': desc, 'Maemo-Upgrade-Description': 'Upgrade description: %s' % desc})

# Test package with tags
desc2 = '<h1>Package</h1> with tags in description field.\n <tt>Teletype text</tt> <i>Italic text</i> <b>Bold text</b> <big>Big text</big> <small>Small text</small> <iframe src ="html_intro.asp" width="100%" height="300"><p>Does your browser support iframes?</p></iframe> <a href="http://www.nokia.com">Click here</a>'
add(P, 'hpm-user-test-special-metadata', v=v0, fields={'Description': desc2, 'Maemo-Upgrade-Description': 'Upgrade description: %s' % desc2})
add(P, 'hpm-user-test-special-metadata', v=v1, fields={'Description': desc2, 'Maemo-Upgrade-Description': 'Upgrade description: %s' % desc2})

# Test package with long maintainer data
maint = ' Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed\n  * do eiusmod tempor incididunt ut labore et dolore magna\n  * aliqua. \t <mail.me@email.com>'
add(P, 'hpm-user-test-special-maintainer', v=v0, fields={'Maintainer': maint})

# Test package with fomatted display name
name = '<h1>Fancy</h1> <i>hpm-user-formatted-display-name</i>'
add(P, 'hpm-user-formatted-display-name', v=v0, fields={'Maemo-Display-Name': name})

# Test package with long display name
name2 = '<a href="http://www.google.com">Click here</a>*Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.*'
add(P, 'hpm-user-long-display-name', v=v0, fields={'Maemo-Display-Name': name2})

# Test package with image
name3 = '<img src="file:///usr/share/package-manager-ui-tests/test_img.jpg">Fancy hpm-user-image</img>'
add(P, 'hpm-user-image', v=v0, fields={'Maemo-Display-Name': name3, 'Description': name3})

# Test package with special characters
name4 = 'äöå½!#¤%&/()=?§@£${[]}\~^_:;'
add(P, 'hpm-user-test-special-characters', v=v0, fields={'Maemo-Display-Name': name4})

# Test Chinese localization.  Text is copypasta from
# http://www.herongyang.com/PHP-Chinese/PHP-UTF-8-Chinese-String-Literals.html
name = '这是一份非常间单的说明书'
add(P, 'hpm-user-chinese', v=v0, fields={'Maemo-Display-Name-fi_FI': name})

# "Forced OS Update" packages
add(P, 'hpm-fou-o', v=v0, fields={'Maemo-Flags': 'system-update',
                                  'Section': 'user/system',
                                  'Depends': 'hpm-fou-s1 (= %s), hpm-fou-s2 (= %s)' % (v0,v0)})
add(P, 'hpm-fou-o', v=v1, fields={'Maemo-Flags': 'system-update',
                                  'Section': 'user/system',
                                  'Depends': 'hpm-fou-s1 (= %s), hpm-fou-s2 (= %s)' % (v1,v0),
                                  'Conflicts': 'hpm-fou-l3'})
add(P, 'hpm-fou-o2', v=v0, fields={'Maemo-Flags': 'system-update',
                                   'Section': 'user/system',
                                   'Depends': 'hpm-fou-s1'})
add(P, 'hpm-fou-o2', v=v1, fields={'Maemo-Flags': 'system-update',
                                   'Section': 'user/system',
                                   'Depends': 'hpm-fou-s2'})
add(P, 'hpm-fou-s1', v=v0, fields={'Section': nonuser_section})
add(P, 'hpm-fou-s1', v=v1, fields={'Section': nonuser_section, 'Conflicts': 'hpm-fou-l2 (<< %s)' % v1})
add(P, 'hpm-fou-s2', v=v0, fields={'Section': nonuser_section})
add(P, 'hpm-fou-l1', v=v0, fields={'Section': nonuser_section, 'Depends': 'hpm-fou-s1 (>= %s)' % v0})
add(P, 'hpm-fou-l2', v=v0, fields={'Section': nonuser_section, 'Depends': 'hpm-fou-s1 (>= %s)' % v0})
add(P, 'hpm-fou-l2', v=v1, fields={'Section': nonuser_section, 'Depends': 'hpm-fou-s1 (>= %s)' % v0})
add(P, 'hpm-fou-l3', v=v0, fields={'Section': nonuser_section})
add(P, 'hpm-fou-l4', v=v0, fields={'Section': nonuser_section, 'Depends': 'hpm-fou-s1 (= %s)' % v0})
add(P, 'hpm-fou-l4', v=v1, fields={'Section': nonuser_section, 'Depends': 'hpm-fou-s1 (= %s)' % v1})
add(P, 'hpm-fou-a1', v=v0, fields={'Section': 'user/hidden', 'Depends': 'hpm-fou-s1 (>= %s)' % v0})
add(P, 'hpm-fou-a2', v=v0, fields={'Section': 'user/hidden', 'Depends': 'hpm-fou-s2 (= %s)' % v0})
add(P, 'hpm-fou-a3', v=v0, fields={'Section': 'user/hidden', 'Depends': 'hpm-fou-s1 (= %s)' % v0})
add(P, 'hpm-fou-a3', v=v1, fields={'Section': 'user/hidden', 'Depends': 'hpm-fou-s1 (= %s)' % v1})
add(P, 'hpm-fou-a4', v=v0, fields={'Section': 'user/hidden', 'Depends': 'hpm-fou-l1'})
add(P, 'hpm-fou-a5', v=v0, fields={'Section': 'user/hidden', 'Depends': 'hpm-fou-l2 (>= %s)' % v0})
add(P, 'hpm-fou-a6', v=v0, fields={'Section': 'user/hidden', 'Depends': 'hpm-fou-s1 (= %s)' % v0})
add(P, 'hpm-fou-a7', v=v0, fields={'Section': 'user/hidden', 'Depends': 'hpm-fou-l3'})
add(P, 'hpm-fou-a8', v=v0, fields={'Section': 'user/hidden', 'Depends': 'hpm-fou-l2 (= %s)' % v0})
add(P, 'hpm-fou-a9', v=v0, fields={'Section': 'user/hidden', 'Depends': 'hpm-fou-s1 (>= %s)' % v0})
add(P, 'hpm-fou-a9', v=v1, fields={'Section': 'user/hidden', 'Depends': 'hpm-fou-s1 (>= %s)' % v0})
add(P, 'hpm-fou-a10', v=v0, fields={'Section': 'user/hidden', 'Depends': 'hpm-fou-l4'})
add(P, 'hpm-fou-a11', v=v0, fields={'Section': 'user/hidden', 'Depends': 'hpm-fou-l4 (= %s)' % v0})
add(P, 'hpm-fou-a12', v=v0, fields={'Section': 'user/hidden'})

# User packages depending on OS dependencies
add(P, 'hpm-user-dep-fou-s1', v=v0, fields={'Depends': 'hpm-fou-s1 (= %s)' % v0 })
add(P, 'hpm-user-dep-fou-s1', v=v1, fields={'Depends': 'hpm-fou-s1 (= %s)' % v1 })

# Base version in higher priority repo 'testrepo-other')
add(P, 'hpm-user-update-in-testrepo', v=v1)

for p in P:
    if p.version == v0 or p.version == '0.0.0-1':
        old_packages.append(p)
    else:
        new_packages.append(p)

# Certified packages

old_other_packages = []
new_other_packages = []

add(old_other_packages, 'hpm-user-cert', v=v0)
add(new_other_packages, 'hpm-user-cert', v=v1)

add(old_other_packages, 'hpm-user-cert2', v=v0)
add(new_other_packages, 'hpm-user-cert2', v=v1)
add(new_packages, 'hpm-user-cert2', v=v2) # This should be ignored since it is in the wrong domain

add(old_packages, 'hpm-user-update-to-cert', v=v0)
add(new_other_packages, 'hpm-user-update-to-cert', v=v1)

# This should be rejected since it is signed with the wrong key
add(old_other_packages, 'hpm-user-wrongsig', v=v0, key = '9E43D4ED')

# AHA: OS metapackage with a higher version update from nontrusted
# repo.
add(old_other_packages, 'hpm-user-conflicts-system4', v=v0,
    fields={'Priority': 'Important',
            'Conflicts': 'hpm-system4',
            'Replaces': 'hpm-system4'})
add(old_other_packages, 'hpm-user-update-in-testrepo', v=v0)
add(old_other_packages, 'hpm-system-meta-other', v=v0,
    fields={'Priority': 'required',
            'Maemo-Flags': 'system-update',
            'Section': 'user/system',
            'Depends': 'hpm-system1 (= %s), hpm-user-conflicts-system4 (= %s)' % (v0, v0),
            'Conflicts': 'hpm-system3',
            'Replaces': 'hpm-system3'})
add(new_other_packages, 'hpm-system-meta-other', v=v1,
    fields={'Priority': 'required',
            'Maemo-Flags': 'system-update',
            'Section': 'user/system',
            'Depends': 'hpm-user-update-in-testrepo (>= %s), hpm-system1 (= %s), hpm-system2 (= %s), hpm-system4 (= %s)' % (v0, v0, v1, v2)})

#2010-11-17 IH: Keeping the trusted and non-trusted test repositories separate from other MW test data, to keep things 'simple'.
old_trusted_repo_packages = []
old_non_trusted_repo_packages = []
new_trusted_repo_packages = []
new_non_trusted_repo_packages = []

add(old_trusted_repo_packages, 'hpm-user-trusted', v=v0)
add(new_trusted_repo_packages, 'hpm-user-trusted', v=v1)
add(old_trusted_repo_packages, 'hpm-user-in-both', v=v0)
add(new_trusted_repo_packages, 'hpm-user-in-both', v=v1)
add(old_non_trusted_repo_packages, 'hpm-user-in-both', v=v0)
add(new_non_trusted_repo_packages, 'hpm-user-in-both', v=v1)
add(old_non_trusted_repo_packages, 'hpm-user-non-trusted', v=v0)
add(new_non_trusted_repo_packages, 'hpm-user-non-trusted', v=v1)
add(old_trusted_repo_packages, 'hpm-user-upgrade-in-non-trusted', v=v0)
add(new_non_trusted_repo_packages, 'hpm-user-upgrade-in-non-trusted', v=v1)
add(new_trusted_repo_packages, 'hpm-user-update-in-trusted', v=v1)
add(old_non_trusted_repo_packages, 'hpm-user-update-in-trusted', v=v0)
add(old_trusted_repo_packages, 'hpm-user-update-in-both', v=v0)
add(new_trusted_repo_packages, 'hpm-user-update-in-both', v=v1)
add(new_non_trusted_repo_packages, 'hpm-user-update-in-both', v=v1)

add(old_trusted_repo_packages, 'hpm-user-trusted-dep', v=v0)
add(new_trusted_repo_packages, 'hpm-user-trusted-dep', v=v1)
add(old_non_trusted_repo_packages, 'hpm-user-non-trusted-dep', v=v0)
add(new_non_trusted_repo_packages, 'hpm-user-non-trusted-dep', v=v1)
add(old_trusted_repo_packages, 'hpm-user-trusted-dep2', v=v0)
add(new_trusted_repo_packages, 'hpm-user-trusted-dep2', v=v1)
add(old_non_trusted_repo_packages, 'hpm-user-non-trusted-dep2', v=v0)
add(new_non_trusted_repo_packages, 'hpm-user-non-trusted-dep2', v=v1)
add(old_trusted_repo_packages, 'hpm-user-trusted-depends-on-non-trusted', v=v0, fields={'Depends': 'hpm-user-non-trusted-dep (= %s)' % v0})
add(old_non_trusted_repo_packages, 'hpm-user-non-trusted-depends-on-trusted', v=v0, fields={'Depends': 'hpm-user-trusted-dep (= %s)' % v0})
add(new_non_trusted_repo_packages, 'hpm-user-non-trusted-depends-on-trusted', v=v1, fields={'Depends': 'hpm-user-trusted-dep'})


add(old_trusted_repo_packages, 'hpm-user-trusted-depends-both', v=v0, fields={'Depends': 'hpm-user-non-trusted-dep (= %s), hpm-user-trusted-dep (= %s)' % (v0, v0)})
add(old_trusted_repo_packages, 'hpm-user-trusted-depends-both2', v=v0, fields={'Depends': 'hpm-user-non-trusted-dep2, hpm-user-trusted-dep2'})
add(old_non_trusted_repo_packages, 'hpm-user-non-trusted-depends-both', v=v0, fields={'Depends': 'hpm-user-non-trusted-dep (= %s), hpm-user-trusted-dep (= %s)' % (v0, v0)})
add(new_non_trusted_repo_packages, 'hpm-user-non-trusted-depends-both', v=v1, fields={'Depends': 'hpm-user-non-trusted-dep, hpm-user-trusted-dep'})


