package util;
use strict;
require Exporter;
our @ISA = qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw( vprint run run_or_die print_file __FUNC__ setup_installed pkgmgr _find_catalogue _get_catalogue_status my_die _list_catalogues run_get_output setup_upgrade teardown get_testdir) ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

my $bin = '../cli/pkgmgr';
if ( ! -e $bin )
{
    $bin = '/usr/bin/pkgmgr';
}
my $verbosity = 0;
my @testdir;
my @tmpout;
my %cats = ();

sub vprint
{
    my ($level, $str) = @_;

    if ($level <= $verbosity) {
	print "$str\n";
    }
}

sub run
{
    my ($cmd, $errmsg) = @_;
    if (! $errmsg) {
	$errmsg = "Error: $cmd failed";
    }

    my @out = `$cmd 2>&1`;
    if ($?) {
	print STDERR "$errmsg\n@out";
	return 0;
    }
    vprint(1, @out);
    return 1;
}

sub run_or_die
{
    my ($cmd, $errmsg) = @_;
    if (! $errmsg) {
	$errmsg = "Error: $cmd failed";
    }

    my @out = `$cmd 2>&1`;
    if ($?) {
	print STDERR "$errmsg\n@out";
	die;
    }
    vprint(1, @out);
}

sub print_file
{
    my ($filename, $contents) = @_;

    open(FILE, ">$filename") || die;
    print(FILE $contents);
    close(FILE);
}

sub __FUNC__  { (caller(1))[3] }

sub my_die
{
    my $function = $_[0];
    my $line = $_[1];
    my $s = $_[2];
#     $s =~ s/\n/\\n/g;
    die "Test $function failed: $s at line $line\n";
}

sub run_get_output
{
    my $cmd = shift;
    my $out = shift;
    @$out = `$cmd 2>&1`;
    if ($verbosity) {
	print "Command: $cmd\n";
	print "Output:' @$out'\n";
    }
    if ($?) {
	print STDERR "error: $cmd\n@$out";
	return $? >> 8;
    }

    return 0;
}

sub pkgmgr
{
    my $args = shift;

    return run_get_output("$bin $args", @_);
}
sub get_testdir()
{
    if (&run_get_output('sh -c \'source testvars.sh; echo $TESTDIR\'', \@testdir)) {
	my_die(__FUNC__, __LINE__, 'setup_installed');
    }
    chomp(@testdir);
    return @testdir;
}

sub setup_installed
{
    @testdir = get_testdir();
    `./stop_pkgmgrd.sh`; # ignore errors
    run_or_die('dui-sb-session -quiet stop');
    run_or_die('./init-test-install.sh');
    run_or_die('dui-sb-session -quiet start');
}

sub setup_upgrade
{
    if (&run_get_output('sh -c \'source testvars.sh; echo $TESTDIR\'', \@testdir)) {
	die 'setup_upgrade';
    }
    chomp(@testdir);

    `./stop_pkgmgrd.sh`; # ignore errors
    run_or_die('dui-sb-session -quiet stop');
    run_or_die('./init-test-update.sh');
    run_or_die('dui-sb-session -quiet start');
    run_or_die('./start_pkgmgrd.sh');
}

sub teardown
{
    run_or_die('./stop_pkgmgrd.sh');
}

sub _list_catalogues
{
    my $ret = pkgmgr("list-catalogues", \@tmpout);
    $ret == 0 || my_die(__FUNC__, __LINE__, "Return value $ret\nOutput '@tmpout'\n");

    %cats = ();

    foreach (@tmpout) {
	if (/^([^ ]+) \(([^ ]+)\)$/) {
	    $cats{$1} = $2;
	}
    }
}

sub _find_catalogue
{
    my ($name) = @_;

    _list_catalogues();

    if (exists($cats{$name})) {
	return ($name => $cats{$name});
    }
    return ();
}

sub _get_catalogue_status
{
    my ($name) = @_;

    my %cat = _find_catalogue($name);
    exists($cat{$name}) || my_die(__FUNC__, __LINE__);
    return $cat{$name};
}
1;
__END__
