/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include <QCoreApplication>
#include <QDebug>
#include <QStringList>
#include <iostream>
#include <QFile>

#include "timedoperations.h"
#include "updater.h"
#include "packagemanager.h"
#include "packagemanagerpendingcallwatcher.h"

#include "timeddefs.h"

void myMessageOutput(QtMsgType type, const char *msg)
{
    static FILE* fd = NULL;
	if(!fd) {
		fd = fopen(APP_LOG, "a");
		fprintf(fd, "\n\nNew run\n\n");
	}
    switch (type) {
    case QtDebugMsg:
        fprintf(fd, "Debug: %s\n", msg);
        break;
    case QtWarningMsg:
        fprintf(fd, "Warning: %s\n", msg);
        break;
    case QtCriticalMsg:
        fprintf(fd, "Critical: %s\n", msg);
        break;
    case QtFatalMsg:
        fprintf(fd, "Fatal: %s\n", msg);
        abort();
    }
	fflush(fd);
}

int main(int argc, char *argv[])
{
    qInstallMsgHandler(myMessageOutput);

    QCoreApplication a(argc, argv);
    a.setApplicationName("timedtasker");
    
    Updater* updater = NULL;

    // Check the mode we're in
    QStringList arguments = QCoreApplication::arguments();
    if ( arguments.count() > 2 )
    {
        std::cout << "Too many arguments. Check the help timedtasker --help\n";
        return RETURN_VALUE_BAD_ARGS;
    }

    if ( arguments.contains(ARG_CHECK_EVENT_ENTRY) )
    {
        qDebug() << Q_FUNC_INFO << "Checking/adding an refresh-event into the timed.";

        updater = new Updater();
        updater->addEventIfNoneExists();

        delete updater; // This code should never be executed
        return RETURN_VALUE_OK;
    }
    else if ( arguments.contains(ARG_REFRESH_CACHE) )
    {
        qDebug() << Q_FUNC_INFO << "Refreshing the pkgmgrd cache...";

        updater = new Updater();
        updater->refreshCache();

        int r = a.exec();
        delete updater;
        return r;
    } else if ( arguments.contains(ARG_CANCEL_EVENT) ) {
        qDebug() << Q_FUNC_INFO << "Canceling the event";

        updater = new Updater();
        bool ok = updater->cancelEvent();
        qDebug() << Q_FUNC_INFO << "Cancel successful:" << ok;
        delete updater;
        return RETURN_VALUE_OK;
    } else if ( arguments.contains(ARG_HELP) ) {
        // Display help

        QString helpMessage = "\ntimedtasker-help:\n";
        helpMessage += ARG_HELP + " - Display this message\n";
        helpMessage += ARG_CHECK_EVENT_ENTRY + " - Check and/or add an pkgmdrd.refresh-cache -event into the timed\n";
        helpMessage += ARG_REFRESH_CACHE + " - Perform pkgmgrd.refresh-cache -operation\n";
        helpMessage += ARG_CANCEL_EVENT + " - Cancel the event\n";
        std::cout << helpMessage.toStdString();
    } else {
        // Unknown argument
        std::cout << "Unknown argument. See help: timedtasker --help\n";
    }

    return RETURN_VALUE_OK;
}
