/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#include "timedoperations.h"
#include <QMap>
#include <QDBusReply>
#include <QList>

TimedOperations::TimedOperations() : timedInterface(NULL)
{
    timedInterface = new Maemo::Timed::Interface;
    if(!timedInterface)
    {
        //TODO: throw exception
        qDebug() << Q_FUNC_INFO << "Error: couldn't get timed interface!";
    }
}

TimedOperations::~TimedOperations()
{
    if(timedInterface)
    {
        delete timedInterface;
        timedInterface = NULL;
    }
}

bool TimedOperations::checkForEvents(const QString &instanceName, uint *cookieId)
{
    *cookieId = 0;
    if ( ! timedInterface ) {
        qDebug() << Q_FUNC_INFO << "Error: no timed interface";
        return false;
    }

    // Check if the event is already in the timed
    QMap<QString, QVariant> eventAttributes;
    eventAttributes.insert(TIMED_ATTRIBUTE_KEY_APP, instanceName);

    QDBusReply< QList<QVariant> > queryResult = timedInterface->query_sync(eventAttributes);
    if ( !queryResult.isValid() ) {
        qDebug() << Q_FUNC_INFO << "Error while communicating with the timed interface! Error:" << queryResult.error();
        return false;
    }

    // TODO: Some additional checks here? Perhaps the verification for the event attributes..
    QList<QVariant> cookies = queryResult.value();
    bool result = !cookies.isEmpty();

    if ( result ) {
        *cookieId = cookies.at(0).toUInt();
        qDebug() << Q_FUNC_INFO << "Cookie id:" << *cookieId;
    }
    
    return result;
}

bool TimedOperations::addEventToTimed(const Maemo::Timed::Event &event)
{
    if ( ! timedInterface ) {
        qDebug() << Q_FUNC_INFO << "Error: no timed interface";
        return false;
    }
    
    QDBusReply<uint> eventAddResult = timedInterface->add_event_sync(event);
    if ( !eventAddResult.isValid() ) {
        qDebug() << Q_FUNC_INFO << "Could not add timed-event for the pkgmgrd! :( Error:" << eventAddResult.error();
        return false ;
    }
    return true;
}

bool TimedOperations::replaceEventToTimed(const Maemo::Timed::Event &event, const uint cookieId)
{
    if ( ! timedInterface ) {
        qDebug() << Q_FUNC_INFO << "Error: no timed interface";
        return false;
    }

    QDBusReply<uint> eventReplaceResult = timedInterface->replace_event_sync(event, cookieId);
    if ( !eventReplaceResult.isValid() ) {
        qDebug() << Q_FUNC_INFO << "Could not replace timed-event for the pkgmgrd! :( Error:" << eventReplaceResult.error();
        return false ;
    }
    return true;
}

bool TimedOperations::cancelEvent(const uint cookieId)
{
    if ( ! timedInterface ) {
        qDebug() << Q_FUNC_INFO << "Error: no timed interface";
        return false;
    }
    
    QDBusReply<bool> cancelResult = timedInterface->cancel_sync(cookieId);
    if ( !cancelResult.isValid() ) {
        qDebug() << Q_FUNC_INFO << "Could not cancel the event. Error:" << cancelResult.error();
        return false;
    }
    
    return cancelResult.value();
}

